
/**
 * @class Describes Archive plus page (scene)
 * @param options - options for page.
 */
function SceneArchPlus(options) {
	/**
	 * @field
	 * @description option of the page.
	 */
	this.options = options;
};

/**
 * @class [ENUM] Options for displaying.
 */
SceneArchPlus.displayOptions = {
		/**
		 * @field
		 * @description show date
		 * @default 1
		 */
	showDate: 1,
	/**
	 * @field
	 * @description show time
	 * @default 2
	 */
	showTime: 2,
	/**
	 * @field
	 * @description show channel icon
	 * @default 4
	 */
	showChannelIcon: 4
};

/**
 * @function
 * @description this function will be called only once when the scene manager show this scene first time
 * initialize the scene controls and styles, and initialize your variables here 
 * scene HTML and CSS will be loaded before this function is called
 */
SceneArchPlus.prototype.initialize = function () {
	Logger.write("SceneArchPlus.initialize()", "ArchPlus");
	
	//SceneArchPlus.currentLang = AppContext.ClientSettings.Language;
	
	SceneArchPlus.initData();
	SceneArchPlus.loadData();
};

/**
 * @function
 * @description initialize data of the page (scene)
 * @returns {SceneArchPlus.initData}
 */
SceneArchPlus.initData = function () {
	
	//this.channels = null;
	/**
	 * @field
	 * @description identifier of current channel.
	 * @default null
	 */
	this.currentChannelId = null;
	//this.genres = null;
	/**
	 * @field
	 * @description first year of the content.
	 * @default 2007
	 */
	this.firstYear = 2007;
	
	/**
	 * @field
	 * @description index of the active category.
	 * @default null
	 */
	this.activeCategoryIndex = null;
	
	/**
	 * @field
	 * @description pager object.
	 * @default null
	 */
	this.pager = null;
	
	/**
	 * @field
	 * @description count of the programs which visible.
	 * @default null
	 */
	this.visiblePrograms = 9;
	
	/**
	 * @field
	 * @description count of programs on the one download page.
	 */
	this.onPage = this.visiblePrograms * 4;
	
	/**
	 * @field
	 * @description length of the category name.
	 * @default 17
	 */
	this.categoryNameLength = 17;
	
	$("#panelSections").empty();
	$("#panelArchPlusCategories").empty();
	
	/**
	 * @field
	 * @description panel of the popular categories.
	 */
	this.popularCategoriesPanel = new SlidingPanel(0, 0, [], "popularCategory", this.onPopularCategorieSelectionChanged, 40, orientations.vertical, false, 320, "categoriesElement", 1500);
    $('#panelArchPlusCategories').append(this.popularCategoriesPanel.panel);
	this.popularCategoriesPanel.hide();
	
	/**
	 * @field
	 * @description panel of the channels categories.
	 */
	this.channelsCategoriesPanel = new SlidingPanel(0, 0, [], "channelCategory", this.onChannelsCategorieSelectionChanged, 40, orientations.vertical, false, 320, "categoriesElement", 1500);
    $('#panelArchPlusCategories').append(this.channelsCategoriesPanel.panel);
	this.channelsCategoriesPanel.hide();
	
	/**
	 * @field
	 * @description panel of the genres categories.
	 */
	this.genresCategoriesPanel = new SlidingPanel(0, 0, [], "genreCategory", this.onGenresCategorieSelectionChanged, 40, orientations.vertical, false, 320, "categoriesElement", 1500);
    $('#panelArchPlusCategories').append(this.genresCategoriesPanel.panel);
	this.genresCategoriesPanel.hide();
	
	/**
	 * @field
	 * @description panel of the sections (channels, genres, etc.).
	 */
	this.sectionsPanel = new SlidingPanel(80, 300, [], "section", this.onActiveSectionChanged, 140, orientations.horizontal, false, 1000);
	$('#panelSections').append(this.sectionsPanel.panel);
	
	/**
	 * @class TV channels
	 */
	this.channels = {
			/**
			 * @field 
			 * @description root channel
			 * @default null
			 */
		root: null,
		/**
		 * @field 
		 * @description active channel
		 * @default null
		 */
		active: null,
		/**
		 * @field 
		 * @description  panel of the channels
		 */
		panel: this.channelsCategoriesPanel,
		/**
		 * @field 
		 * @description channels selection history 
		 * @default []
		 */
		history: []
	};
	
	/**
	 * @class genres
	 */
	this.genres = {
			/**
			 * @field 
			 * @description root genre
			 * @default null
			 */
		root: null,
		/**
		 * @field 
		 * @description active genre
		 * @default null
		 */
		active: null,
		/**
		 * @field 
		 * @description  panel of the genres
		 */
		panel: this.genresCategoriesPanel,
		/**
		 * @field 
		 * @description genres selection history 
		 * @default []
		 */
		history: []
	};
};

/**
 * @function 
 * @description Provides translating page (scene) on the language that specified in settings.
 */
SceneArchPlus.translate = function () {
    if (SceneArchPlus.currentLang != AppContext.ClientSettings.Language)
    {
		$('#lblArchPlusTitle').sfLabel({text: Resources.getResources().ArchPlusTitle.toLowerCase()});
		$("#archPlusNoData").html(Resources.getResources().NoData);
		SceneArchPlus.loadSections();
		SceneArchPlus.bindPopularCategories();
		if (SceneArchPlus.channels && SceneArchPlus.channels.root && SceneArchPlus.channels.root.length) {
			SceneArchPlus.updateChannelsSubCategories();
			SceneArchPlus.bindChannelsCategories(SceneArchPlus.channels.root);
			
			if (SceneArchPlus.activePanel == "channelsCategoriesPanel") {
				SceneArchPlus[SceneArchPlus.activePanel].invokeSelectedChangedCallback();
			}
		}
		
        SceneArchPlus.currentLang = AppContext.ClientSettings.Language;
        SceneArchPlus.setHelpBar();
    }
};

/**
 * @function
 * @description Sets line of the help bar.
 */
SceneArchPlus.setHelpBar = function () {
	if (ScenePlayer.isPlaying) {
		$('#keysArchPlus').sfKeyHelp({
			'red':Resources.getResources().LabelSettings,
			'green': Resources.getResources().Search,
			'yellow':Resources.getResources().Info,
			'blue':Resources.getResources().Player, 
			'Stop':Resources.getResources().Stop ,
			'updown':'', 'leftright':Resources.getResources().MoveCursor, 
			'enter':Resources.getResources().Select,
			'return':Resources.getResources().BackToMainMenu
		});
	} else {
		$('#keysArchPlus').sfKeyHelp({
			'red':Resources.getResources().LabelSettings,
			'green': Resources.getResources().Search,
			'yellow':Resources.getResources().Info,
			'blue':Resources.getResources().Refresh,
			'updown':'', 'leftright':Resources.getResources().MoveCursor,
			'enter':Resources.getResources().Select,
			'return':Resources.getResources().BackToMainMenu
		});
	}
};

/**
 * @event 
 * @description Occurs when selected element in popular category changes.
 * @param {string} element - new selected element of the popular category.
 * @param {string} index - element identifier.
 */
SceneArchPlus.onPopularCategorieSelectionChanged = function (element, index) {
	var requestType = null;
	Logger.write("New popular element: " + element, "ArchPlus", true);
	
	var id = element.replace("#divpopularCategory", "");
	
	var category = null;
	for (var i = 0; i < SceneArchPlus.popular.length; i++) {
		if (SceneArchPlus.popular[i].id == id) {
			category = SceneArchPlus.popular[i];
			break;
		}
	}
	
	if (category) {
		SceneArchPlus.currentDisplayOptions = category.displayOptions;
		SceneArchPlus.loadPrograms(category.tag, null, null, null, null);
		SceneArchPlus.activeCategoryIndex = index;
	}
};

/**
 * @event 
 * @description Occurs when selected element in channel category changes.
 * @param {string} element - new selected element of the channel category.
 * @param {string} index - element identifier.
 */
SceneArchPlus.onChannelsCategorieSelectionChanged = function (element, index) {
	Logger.write("New channels element: " + element, "ArchPlus", true);
	if (!element) {
		return;
	}
	
	var id = element.replace("#divchannelCategory", "");
	var categories = SceneArchPlus.channels.active;
	var isChannels = categories[0].isChannel;
	if (isChannels && isChannels === true) {
		SceneArchPlus.currentChannelId = id;
	}
	if (isChannels) {
		SceneArchPlus.currentDisplayOptions = SceneArchPlus.displayOptions.showTime | SceneArchPlus.displayOptions.showDate | SceneArchPlus.displayOptions.showChannelIcon;
		var tag = categories[0].subCategories[0].tag;
		id = parseInt(id);
		
		SceneArchPlus.loadPrograms("SearchByDateRangExpanded", id, tag.dateFrom, tag.dateTill, null);
		SceneArchPlus.activeCategoryIndex = index;
	} else if (SceneArchPlus.currentChannelId) {
		var item = SceneArchPlus.getItem(categories, id);
		if (item && item.tag) {
			var dateFrom = item.tag.dateFrom;
			var dateTill = item.tag.dateTill;
			var displayOptions = item.displayOptions ? item.displayOptions : 7;
			SceneArchPlus.currentDisplayOptions = displayOptions;
			SceneArchPlus.loadPrograms("SearchByDateRangExpanded", SceneArchPlus.currentChannelId, dateFrom, dateTill, null);
			SceneArchPlus.activeCategoryIndex = index;
		}
	}
};

/**
 * @event 
 * @description Occurs when selected element in genres category changes.
 * @param {string} element - new selected element of the genres category.
 * @param {string} index - element identifier.
 */
SceneArchPlus.onGenresCategorieSelectionChanged = function (element, index) {
	Logger.write("New genres element: " + element, "ArchPlus", true);
	
	var genreId = element.replace("#divgenreCategory", "");
	if (genreId) {
		SceneArchPlus.currentDisplayOptions = SceneArchPlus.displayOptions.showDate | SceneArchPlus.displayOptions.showTime | SceneArchPlus.displayOptions.showChannelIcon;
		SceneArchPlus.loadPrograms(null, null, null, null, genreId, null, "name ASC");
		SceneArchPlus.activeCategoryIndex = index;
	}
};

/**
 * @event 
 * @description Occurs when selected section changes.
 * @param {string} element - new selected section.
 */
SceneArchPlus.onActiveSectionChanged = function (element) {
	SceneArchPlus.popularCategoriesPanel.stopTimer();
	SceneArchPlus.channelsCategoriesPanel.stopTimer();
	SceneArchPlus.genresCategoriesPanel.stopTimer();
	
	SceneArchPlus.popularCategoriesPanel.hide();
	SceneArchPlus.channelsCategoriesPanel.hide();
	SceneArchPlus.genresCategoriesPanel.hide();
	
	Logger.write("Focusing element: " + element, "ArchPlus");
	
	if (element.indexOf("popular") > -1) {
		SceneArchPlus.popularCategoriesPanel.show();
		SceneArchPlus.popularCategoriesPanel.invokeSelectedChangedCallback();
	} else if (element.indexOf("channels") > -1) {
		SceneArchPlus.channelsCategoriesPanel.show();
		SceneArchPlus.channelsCategoriesPanel.invokeSelectedChangedCallback();
	} else if (element.indexOf("genres") > -1) {
		SceneArchPlus.genresCategoriesPanel.show();
		SceneArchPlus.genresCategoriesPanel.invokeSelectedChangedCallback();
	}
};

/**
 * @function
 * @description Provides load sections panel.
 */
SceneArchPlus.loadSections = function () {
	var sections = [];
	sections.push(new PanelElement("channels", Resources.getResources().ArchPlusChannels));
	sections.push(new PanelElement("genres", Resources.getResources().ArchPlusGenres));
	sections.push(new PanelElement("popular", Resources.getResources().ArchPlusPopular));
	SceneArchPlus.sectionsPanel.loadData(sections, true);
};

/**
 * @function
 * @description Provides load data on page (scene).
 */
SceneArchPlus.loadData = function () {
	SceneArchPlus.showLoading();
	SceneArchPlus.offset = AppContext.getNaEstUtcOffset();
	setTimeout(function () {
		SceneArchPlus.loadGenres();
		SceneArchPlus.loadChannels();
	}, 1);
};

/**
 * @function
 * @description Provides load genres.
 */
SceneArchPlus.loadGenres = function () {
	var errCallback = function (jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "ArchPlus", true, textStatus);
	};
	JsonServices.content.GetClientContentGenres(
		AppContext.user.sid, "ArcplusGenreType",
		SceneArchPlus.onGenresLoaded, errCallback
	);
};

/**
 * @function
 * @description Provides load channels.
 */
SceneArchPlus.loadChannels = function () {
	var errCallback = function (jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "ArchPlus", true, textStatus);
	};
	JsonServices.content.GetClientChannels(
		AppContext.user.sid, "ArcPlus", null, null,
		SceneArchPlus.onChannelsLoaded, errCallback
	);
};

/**
 * @function
 * @description Provides load programs
 * @param {number} requestType - Defines search range for Video On Demand content (ArcPlus or VOD content type). ({@link JsonServices.OnDemandRequestType}).
 * @param {number} channelId - Requested items channel ID
 * @param {string} dateFrom - Date of an item. Date when an item was added to the video collection.
 * @param {string} dateTill - Upper limit of item date for diapason search
 * @param {number} genreId - Genre id.
 * @param {string} keyWord - A word to look for in item names (i.e. names of films, TV programs, series etc.)
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @returns {SceneArchPlus.loadPrograms}
 */
SceneArchPlus.loadPrograms = function (requestType, channelId, dateFrom, dateTill, genreId, keyWord, orderBy) {
	var from = null;
    var till = null;
	if (dateFrom && dateTill) {
		from = new Date(dateFrom);
		till = new Date(dateTill);
		from.setHours(from.getHours() - SceneArchPlus.offset + AppContext.deviceOffset/60);
		till.setHours(till.getHours() - SceneArchPlus.offset + AppContext.deviceOffset/60);
	}
	
	var request = new Models.ArchPlusRequest(requestType, channelId, from, till, genreId, keyWord, orderBy);
	
	if (!request.equals(this.request)) {
		this.showLoading();
		var errCallback = function (jqXHR, textStatus, error) {
			ErrorHandler.HandleError(error, "ArchPlus", true, textStatus);
        };
		
		Logger.write("DateFrom: " + dateFrom + "; DateTill: " + dateTill, "ArchPlus", true);
		JsonServices.content.GetClientProgramGuide(
			AppContext.user.sid, "ArcPlus", requestType, channelId, genreId,
			from, till, keyWord, null, false, this.onPage, 1,
			AppContext.ClientSettings.shiftTimeZoneName, AppContext.ClientSettings.timeZoneName,
			SceneArchPlus.onProgramsLoaded, errCallback
		);
		/**
		 * @field 
		 * @description request string.
		 */
		this.request = request;
	}
};

/**
 * @function
 * @description Checks data was loaded or not. If true - occurs onDataLoaded().
 */
SceneArchPlus.checkDataLoaded = function () {
	if (SceneArchPlus.channels.root && SceneArchPlus.genres.root) {
		SceneArchPlus.onDataLoaded();
	}
};

/**
 * @event
 * @description Occurs when data for the page (scene) was successfully loaded.
 */
SceneArchPlus.onDataLoaded = function () {
	if (this.channels.root.length > 0 && this.genres.root.length > 0) {
	//Logger.write("data loaded from: " + (new Error()).stack, "ArchPlus", true);
		$("#archPlusNoData").hide();
		this.bindChannelsCategories(this.channels.root);
		this.bindGenresCategories(this.genres.root);
		
		this.focusActiveCategories();
		this.sectionsPanel.invokeSelectedChangedCallback();
	} else {
		SceneArchPlus.popularCategoriesPanel.stopTimer();
		SceneArchPlus.channelsCategoriesPanel.stopTimer();
		SceneArchPlus.genresCategoriesPanel.stopTimer();
		$("#archPlusNoData").show();
		$("#panelSections").hide();
		SceneArchPlus.hideLoading();
	}
};

/**
 * @event
 * @description Occurs when genres was successfully loaded.
 * @param {object} result
 */
SceneArchPlus.onGenresLoaded = function (result) {
	var ts = new Date();
	var genres = Models.ContentGenresTree(result.genres);
	Logger.write("Genres generated in '" + (new Date() - ts) + "' ms", "ArchPlus", true);
	SceneArchPlus.genres.root = genres;
	
	SceneArchPlus.checkDataLoaded();
};

/**
 * @event
 * @description Occurs when channels was successfully loaded.
 * @param {object} result
 */
SceneArchPlus.onChannelsLoaded = function (result) {
	var channels = Models.CategoriesFromChannelsCollection(result.items);
	SceneArchPlus.channels.root = channels;
	SceneArchPlus.updateChannelsSubCategories(SceneArchPlus.checkDataLoaded);
};

/**
 * @function
 * @description Provides update of the channel's sub categories.
 * @param {function} callback - occurs when update success.
 */
SceneArchPlus.updateChannelsSubCategories = function (callback) {
	var getTimeCallback = function (result) {
		var tmpDate = result;
		var shift = 0;
		for (var i = 0; i < AppContext.TimeZones.length; i++) {
			if (AppContext.ClientSettings.timeZoneName == AppContext.TimeZones[i].id) {
				shift = parseInt(AppContext.TimeZones[i].utcOffset);
				break;
			}
		}
		tmpDate.setMinutes(tmpDate.getMinutes() + shift);
		var time = Helpers.getUtcTime(tmpDate);
		
		var channelSubCategories = SceneArchPlus.createChannelSubCategories(time);
		for (var i = 0; i < SceneArchPlus.channels.root.length; i++) {
			SceneArchPlus.channels.root[i].subCategories = channelSubCategories;
		}

		if (!!callback) {
			callback();
		}
	};

	var errCallback = function (jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "LiveTv", true, textStatus, jqXHR);
	};
	
	JsonServices.media.GetCurrentUtcTime(getTimeCallback, errCallback);
};

/**
 * @event
 * @description Occurs when programs is loaded.
 * @param {object} result.
 */
SceneArchPlus.onProgramsLoaded = function (result) {
	var isChannels = SceneArchPlus.activePanel == "channelsCategoriesPanel";
	var paging = result.paging;
	Logger.write("Total items: " + paging.totalItems, "ArchPlus", true);
	SceneArchPlus.pager = new ListPager(paging, "#lbArchPlusPrograms", SceneArchPlus.visiblePrograms, SceneArchPlus.onPageChanging, SceneArchPlus.onPagerOutOfBounds);
	SceneArchPlus.programs = result.items;
	
	SceneArchPlus.bindPrograms();
	if (SceneArchPlus.activePanel == "programs") {
		if (SceneArchPlus.programs.length > 0) {
			SceneArchPlus.focusPrograms();
		} else {
			SceneArchPlus.focusActiveCategories();
		}
	}
	SceneArchPlus.hideLoading();
};

/**
 * @event
 * @description Occurs when page is changing.
 * @param {number} onPage - count of the elements on one page
 * @param {number} pageNumber - current page number
 * @param {function} pageChangedCallback - occurs when page was changed.
 */
SceneArchPlus.onPageChanging = function (onPage, pageNumber, pageChangedCallback) {
	SceneArchPlus.showLoading();
	var completeCallback = function (result) {
		var isChannels = SceneArchPlus.activePanel == "channelsCategoriesPanel";
		SceneArchPlus.programs = result.items;
		
		SceneArchPlus.bindPrograms();
		pageChangedCallback(true);
		Logger.write("New scroll page number: " + SceneArchPlus.pager.scroll.pageNumber, "ArchPlus", true);
		$('#scrollArchPlusPrograms').sfScroll('move', SceneArchPlus.pager.scroll.pageNumber);
		SceneArchPlus.hideLoading();
	};
	
	JsonServices.content.GetClientProgramGuide(
		AppContext.user.sid, "ArcPlus", SceneArchPlus.request.requestType, SceneArchPlus.request.channelId, 
		SceneArchPlus.request.genreId, SceneArchPlus.request.dateFrom, SceneArchPlus.request.dateTill, 
		SceneArchPlus.request.keyWord, SceneArchPlus.request.orderBy, false, onPage, pageNumber,
		AppContext.ClientSettings.shiftTimeZoneName, AppContext.ClientSettings.timeZoneName,
		completeCallback,
		function (jqXHR, textStatus, error) {
			ErrorHandler.HandleError(error, "ArchPlus", true, textStatus);
			pageChangedCallback(false);
		}
	);
};

/**
 * @event
 * @description Occurs when page is out of bounds
 * @param num - page number.
 */
SceneArchPlus.onPagerOutOfBounds = function (num) {
	if (num < 0) {
		$('#lbArchPlusPrograms').sfList('blur');
		SceneArchPlus.focusSectionsPanel();
	}
};

/**
 * @function
 * @description Gets html-code of the program element.
 * @param {object} program - program element
 * @returns {string} html-code.
 */
SceneArchPlus.getProgramHtml = function (program) {
	var result = "";
	var isChannels = !!SceneArchPlus.request.channelId;
	var date = new Date(program.recentAirTime.startTime);
	date.setHours(date.getHours() + SceneArchPlus.offset - AppContext.deviceOffset/60);
	if (SceneArchPlus.currentDisplayOptions & SceneArchPlus.displayOptions.showDate) {
		result += "<span class='programDate'>" + DateHelper.getArchPlusDate(date) + "</span>";
	}
	if (SceneArchPlus.currentDisplayOptions & SceneArchPlus.displayOptions.showChannelIcon) {
		var channelId = isChannels ? SceneArchPlus.currentChannelId : program.recentAirTime.channelID;
		var url = AppContext.ImageUrlTemplate.replace("{0}", channelId).replace("{1}", 25).replace("{2}", 1) + "&width=28";
		result += "<div class='logoBack'><img  width='28' height='28' src='" + url + "' /></div>";
	}
	if (SceneArchPlus.currentDisplayOptions & SceneArchPlus.displayOptions.showTime) {
		result += "<span class='programTime'>" + DateHelper.getTime(date) + "</span>";
	}
	var titleClass = "programTitle";
	var hasSeries = program.isContainer && program.episodeNum === 1 || program.episodeNum > 1;
	if (hasSeries && !isChannels) {
		titleClass += " series";
	}
	var programName = program.name.length > 24 ? program.name.substring(0, 21) + "..." : program.name;
	result += "<span class='" + titleClass + "'>" + programName + "</span>";

	return result;
};

/**
 * @function
 * @description bind popular categories panel.
 * @returns {SceneArchPlus.bindPopularCategories}
 */
SceneArchPlus.bindPopularCategories = function () {
	var popular = [];
	var displayOptions = SceneArchPlus.displayOptions.showDate | SceneArchPlus.displayOptions.showTime | SceneArchPlus.displayOptions.showChannelIcon;
	popular.push(new Models.Category("thisWeek", Resources.getResources().ArchPlusWeek, false, null, "TopHundredWatchedWeek", displayOptions));
	popular.push(new Models.Category("thisMonth", Resources.getResources().ArchPlusMonth, false, null, "TopHundredWatchedMonth", displayOptions));
	popular.push(new Models.Category("allTime", Resources.getResources().ArchPlusAllTime, false, null, "TopHundredWatched", displayOptions));
	this.popular = popular;

	var elements = [];
	for (var i = 0; i < this.popular.length; i++) {
		elements[i] = new PanelElement(this.popular[i].id, this.popular[i].name);
	}
	
	this.popularCategoriesPanel.loadData(elements, true);
};

/**
 * @function
 * @description Bind categories panel
 * @param {object} section - category section.
 * @param {Array} categories - content category array.
 * @param {string} idToFocus - identifier of the element for focusing.
 */
SceneArchPlus.bindCategories = function (section, categories, idToFocus) {
	if (!categories || !categories.length || !section || !section.root || !section.root.length) {
		return;
	}
	var elements = [];
	var index = 0;
	for (var i = 0; i < categories.length; i++) {
		var name = categories[i].hasSubCategories ? categories[i].name + ">" : categories[i].name;
		elements[i] = new PanelElement(categories[i].id, name);
		if (idToFocus && categories[i].id == idToFocus) {
			index = i;
		}
	}
	section.active = categories;
	Logger.write("Binding categories, elements length: " + elements.length, "ArchPlus", true);
	section.panel.loadData(elements);
	if (idToFocus) {
		section.panel.setIndex(index);
	}
};

/**
 * @function
 * @description Bind categories of the channels panel
 * @param {Array} categories - content category array.
 * @param {string} idToFocus - identifier of the element for focusing.
 */
SceneArchPlus.bindChannelsCategories = function (categories, idToFocus) {
	this.bindCategories(this.channels, categories, idToFocus);
};

/**
 * @function
 * @description Bind categories of the genres panel.
 * @param {Array} categories - content category array.
 * @param {string} idToFocus
 */
SceneArchPlus.bindGenresCategories = function (categories, idToFocus) {
	this.bindCategories(this.genres, categories, idToFocus);
};

/**
 * @function
 * @description Bind programs panel.
 */
SceneArchPlus.bindPrograms = function () {
	var len = this.programs.length;
	if (len > 0) {
		$('#lbArchPlusPrograms').css('visibility', 'visible');

		if (len > this.visiblePrograms)
		{
			$('#scrollArchPlusPrograms').css('visibility', 'visible');
		}
		
		var lbElems = [];
		for (var i = 0; i < len; i++) {
			lbElems.push(this.getProgramHtml(this.programs[i]));
		}
		var perPage = len > this.visiblePrograms ? this.visiblePrograms : len;
		$('#lbArchPlusPrograms').sfList({data: lbElems, itemsPerPage: perPage});
		$('#lbArchPlusPrograms').sfList('move', 0);
		$('#lbArchPlusPrograms').sfList('blur');
		var pagesNum = SceneArchPlus.pager.scroll.getTotalPages();
		Logger.write("Total items: " + SceneArchPlus.pager.paging.totalItems + "; total pages: " + pagesNum, "ArchPlus", true);
		$('#scrollArchPlusPrograms').sfScroll({pages: pagesNum});
		$("#panelArchPlusPrograms").show();
		$("#scrollArchPlusPrograms").show();
		$("#pagerInfo").html(this.pager.paging.getPagerInfo());
	} else {
		$('#scrollArchPlusPrograms').css('visibility', 'hidden');
		$('#lbArchPlusPrograms').sfList('clear');
		$('#lbArchPlusPrograms').css('visibility', 'hidden');
		$("#pagerInfo").html("");
	}
	if (AppContext.CurrentScene != "ArchTv") {
		SceneArchPlus.sceneWasntActive = true;
	}
	Logger.write("Programs binded", "ArchPlus", true);
};

/**
 * @function
 * @description Provides creating sub categories of the channel.
 * @param {string} time - current time.
 * @returns {Array} sub categories.
 */
SceneArchPlus.createChannelSubCategories = function (time) {
	var defaultDisplayOptions = SceneArchPlus.displayOptions.showTime | SceneArchPlus.displayOptions.showDate | SceneArchPlus.displayOptions.showChannelIcon;

	var now = new Date(time);
	var subCategories = [];
	var dateTill = new Date(time);
	var dateFrom = new Date(dateTill.getTime());
	dateFrom.setHours(dateFrom.getHours() - 24);
	var lastDayTag = {
		dateFrom: dateFrom,
		dateTill: dateTill
	};
	subCategories.push(new Models.Category("lastDay", Resources.getResources().LastDay, false, null, lastDayTag, defaultDisplayOptions));
	
	var weekDay = now.getDay();
	weekDay = weekDay == 0 ? 7 : weekDay;
	if (weekDay > 1) {
		var lastWeekFrom = new Date(now.getFullYear(), now.getMonth(), now.getDate() - weekDay, 0, 0, 0);
		var lastWeekTag = new DateHelper.DateRange(lastWeekFrom, now);
		var lastWeek = new Models.Category("lastWeek", Resources.getResources().ArchPlusWeek, true, null, lastWeekTag, defaultDisplayOptions);
		for (var i = 1; i < weekDay; i++) {
			var tmpDate = new Date(now.getFullYear(), now.getMonth(), now.getDate() - i, now.getHours(), now.getMinutes(), now.getSeconds());
			var tag = new DateHelper.DateRange(tmpDate);
			lastWeek.subCategories.push(new Models.Category("lastWeekDay"+i, Resources.getResources()["ArchPlusDay"+i], false, lastWeek, tag, defaultDisplayOptions));
		}
		subCategories.push(lastWeek);
	}
	
	var lastMonthFrom = new Date(now.getFullYear(), now.getMonth(), 1, 0, 0, 0);
	var lastMonthTag = new DateHelper.DateRange(lastMonthFrom, now);
	var lastMonth = new Models.Category("lastMonth", Resources.getResources().ArchPlusMonth, true, null, lastMonthTag, defaultDisplayOptions);
	var day = now.getDate();
	for (var i = day; i > 0; i--) {
		now.setDate(i);
		var tag = new DateHelper.DateRange(now);
		lastMonth.subCategories.push(new Models.Category("lastMonthDay"+i, i.toString(), false, lastMonth, tag, defaultDisplayOptions));
	}
	subCategories.push(lastMonth);
	
	var date = new Date(time);
	var yearsToShow = date.getFullYear() - this.firstYear + 1;
	for (var i = 0; i < yearsToShow; i++) {
		var yearStr = date.getFullYear().toString();
		var dateFrom = new Date(date.getFullYear(), 0, 1, 0, 0, 0);
		var yearTag = new DateHelper.DateRange(dateFrom, date);
		var year = new Models.Category("year" + yearStr, yearStr, true, null, yearTag, defaultDisplayOptions);
		
		var months = date.getMonth() + 1;
		for (var j = 0; j < months; j++) {
			var monthStr = date.getMonth().toString();
			dateFrom = new Date(date.getFullYear(), date.getMonth(), 1, 0, 0, 0);
			var monthTag = new DateHelper.DateRange(dateFrom, date);
			var month = new Models.Category("year"+yearStr+"month"+monthStr, Resources.getResources()["ArchPlusMonth"+monthStr], true, year, monthTag, defaultDisplayOptions);
			var daysInMonth = date.getDate();
			for (var k = 0; k < daysInMonth; k++) {
				var dayStr = date.getDate().toString();
				var dayTag = new DateHelper.DateRange(date);
				var monthDay = new Models.Category("year"+yearStr+"month"+monthStr+"day"+dayStr, dayStr, false, month, dayTag, defaultDisplayOptions);
				
				month.subCategories.push(monthDay);
				date.setDate(date.getDate() - 1);
			}
			
			year.subCategories.push(month);
		}
		
		subCategories.push(year);
	}
	
	return subCategories;
};

/**
 * @function 
 * @description Provides displaying loading element.
 */
SceneArchPlus.showLoading = function () {
	Logger.write("SceneArchPlus.showLoading()", "ArchPlus");
    SceneArchPlus.isLoading = true;
	if (AppContext.CurrentScene == "ArchPlus") {
		Helpers.showLoading();
	}
    SceneArchPlus.dataLoaded = false;
};

/**
 * @function 
 * @description Provides hiding loading element.
 */
SceneArchPlus.hideLoading = function () {
	Logger.write("SceneArchPlus.hideLoading()", "ArchPlus");
	SceneArchPlus.isLoading = false;
	if (AppContext.CurrentScene == "ArchPlus") {
		Helpers.hideLoading();
	}
    SceneArchPlus.dataLoaded = true;
};

/**
 * @function
 * @description Provides sets focus on sections panel.
 */
SceneArchPlus.focusSectionsPanel = function () {
	SceneArchPlus.sectionsPanel.focusPanel();
	SceneArchPlus.activePanel = "sectionsPanel";
};

/**
 * @function
 * @description Provides sets focus on popular categories panel.
 */
SceneArchPlus.focusPopular = function () {
	SceneArchPlus.popularCategoriesPanel.focusPanel();
	SceneArchPlus.activePanel = "popularCategoriesPanel";
};

/**
 * @function
 * @description Provides sets focus on channels categories panel.
 */
SceneArchPlus.focusChannels = function () {
	SceneArchPlus.channelsCategoriesPanel.focusPanel();
	SceneArchPlus.activePanel = "channelsCategoriesPanel";
};

/**
 * @function
 * @description Provides sets focus on genres categories panel.
 */
SceneArchPlus.focusGenres = function () {
	SceneArchPlus.genresCategoriesPanel.focusPanel();
	SceneArchPlus.activePanel = "genresCategoriesPanel";
};

/**
 * @function
 * @description Provides sets focus on active category in sections panel.
 */
SceneArchPlus.focusActiveCategories = function () {
	var focusedId = this.sectionsPanel.getFocusedId();
	if (focusedId.indexOf("popular") > -1) {
		this.focusPopular();
	} else if (focusedId.indexOf("channels") > -1) {
		this.focusChannels();
	} else if (focusedId.indexOf("genres") > -1) {
		this.focusGenres();
	}
};

/**
 * @function
 * @description Provides sets focus on programs panel.
 */
SceneArchPlus.focusPrograms = function () {
	$("#lbArchPlusPrograms").sfList('focus');
	SceneArchPlus.activePanel = "programs";
	$("#pagerInfo").html(this.pager.paging.getPagerInfo());
};

/**
 * @function
 * @description Provides gets childrens of the items.
 * Iterates items while item id != incoming id. if no one coincidence will be found - returns incoming items.
 * @param {Array} items - items array.
 * @param {string} id - identifier of the element, in which are looking for children.
 * @returns {Array} childrens or incoming array.
 */
SceneArchPlus.getChildren = function (items, id) {
	for (var i = 0; i < items.length; i++) {
		if (items[i].id == id) {
			return items[i].subCategories;
		}
	}
	return items;
};

/**
 * @function 
 * @description Gets item by id from items array
 * @param {Array} items - incoming array.
 * @param {string} id - element identifier. 
 * @returns {object} finded item or null.
 */
SceneArchPlus.getItem = function (items, id) {
	for (var i = 0; i < items.length; i++) {
		if (items[i].id == id) {
			return items[i];
		}
	}
	
	return null;
};

/**
 * @function
 * @description Gets categories parent.
 * @param {object} categories
 * @returns {object} parent or null.
 */
SceneArchPlus.getParentCategories = function (categories) {
	if (categories && categories.active) {
		var parent = categories.active[0].parent;
		if (parent) {
			if (parent.parent) {
				return parent.parent.subCategories;
			} else {
				return categories.root;
			}
		}
	}
	
	return null;
};

/**
 * @function
 * @description Marks movie as bookmarked or not bookmared 
 * @param {number} id - movie's identifier
 * @param {boolean} status - bookmarked or not bookmared.
 */
SceneArchPlus.changeFavorite = function (id, status) {
	if (!SceneArchPlus.programs || SceneArchPlus.programs.length == 0) {
		return;
	}
	
	for (var i = 0; i < SceneArchPlus.programs.length; i++) {
		var program = SceneArchPlus.programs[i];
		if (program.id == id) {
			program.bookmarked = status;
			break;
		}
	}
};

/**
 * @function
 * @description Gets channel aspect ratio
 * @param {object} channelId - channels' identifier
 * @returns {object} aspect ratio or null.
 */
SceneArchPlus.getChannelAr = function (channelId) {
	if (!SceneArchPlus.channels || !SceneArchPlus.channels.root) {
		if (channelId == 23) {
			return {
				x: 16,
				y: 9
			};
		}
		return null;
	}
	
	for (var i = 0; i < SceneArchPlus.channels.root.length; i++) {
		var channel = SceneArchPlus.channels.root[i];
		if (channel.id == channelId) {
			return channel.ar;
		}
	}
	
	if (channelId == 23) {
		return {
			x: 16,
			y: 9
		};
	}
	return null;
};

/**
 * @function
 * @description Provides page refresh.
 */
SceneArchPlus.refresh = function () {
	SceneArchPlus.showLoading();
	//$("#panelArchPlusPrograms").hide();
	$("#lbArchPlusPrograms").sfList("clear");
	$("#lbArchPlusPrograms").html("");
	$("#scrollArchPlusPrograms").hide();
	SceneArchPlus.initData();
	SceneArchPlus.loadSections();
	SceneArchPlus.bindPopularCategories();
	SceneArchPlus.loadData();
};

/**
 * @function
 * @description Provides opening of the mediaplayer
 */
SceneArchPlus.openPlayer = function () {
	var itemIndex = $('#lbArchPlusPrograms').sfList('getIndex');
	var item = SceneArchPlus.programs[itemIndex];
	
	var callback = function (response) {
		try {
			sf.scene.hide(AppContext.CurrentScene);
			sf.scene.show('Player');
			var video = item;
			video.currentTime = 0;
			video.url = response.URL;
			video.playlist = response.Playlist;
			video.ar = Helpers.parseAr(response.DAR);
			video.contentType = "ArcPlus";
			ScenePlayer.setContent(video, "ArchPlus");
			sf.scene.focus('Player');
		} catch (error) {
			ErrorHandler.HandleError(error, "ArchPlus");
		}
    };
	var errCallback = function (jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "ArchPlus", true, textStatus);
	};
	
    try {
		JsonServices.media.GetClientStreamUri(
			AppContext.user.sid, "ArcPlus", item.id,
			AppContext.ClientSettings.bAId, AppContext.ClientSettings.cdnId,
			AppContext.ClientSettings.qualityPreset, AppContext.ClientSettings.shiftTimeZoneName,
			callback, errCallback
		);
	} catch (error) {
        ErrorHandler.HandleError(error, "ArchPlus");  
	}
};

/**
 * @function
 * @description Provides moving to previous category
 * @param {object} section - category section.
 */
SceneArchPlus.moveToPrevCategories = function (section) {
	var categories = this.getParentCategories(section);
	
	var id = null;
	if (section.active[0].parent) {
		id = section.active[0].parent.id;
	}
	
	Logger.write("Sliding to init position", "ArchPlus", true);
	this.bindCategories(section, categories, id);
	section.panel.slide(section.history.pop(), true);
	section.panel.focusPanel();
	if (categories) {
		return true;
	}
};

/**
 * @event
 * @description Occurs when left button pressed 
 */
SceneArchPlus.handleLeft = function () {
	if (this.activePanel == "sectionsPanel") {
		this.sectionsPanel.focusPrev();
	} else if (this.activePanel == "genresCategoriesPanel") {
		return this.moveToPrevCategories(this.genres);
	} else if (this.activePanel == "channelsCategoriesPanel") {
		return this.moveToPrevCategories(this.channels);
	} else if (this.activePanel == "programs") {
		$('#lbArchPlusPrograms').sfList('blur');
		this.focusActiveCategories();
		return true;
	}
};

/**
 * @event
 * @description Occurs when right button pressed 
 */
SceneArchPlus.handleRight = function () {
	if (this.activePanel == "sectionsPanel") {
		this.sectionsPanel.focusNext();
	} else if (this.activePanel == "popularCategoriesPanel" || this.activePanel == "channelsCategoriesPanel" || this.activePanel == "genresCategoriesPanel") {
		if (this.programs.length > 0) {
			this[this.activePanel].blurPanel();
			this.focusPrograms();
		}
	}
};

/**
 * @event
 * @description Occurs when up button pressed 
 */
SceneArchPlus.handleUp = function () {
	if (this.activePanel == "popularCategoriesPanel" || this.activePanel == "channelsCategoriesPanel" || this.activePanel == "genresCategoriesPanel") {
		if (this[this.activePanel].index > 0) {
			this[this.activePanel].focusPrev();
		} else {
			this[this.activePanel].setIndex(this.activeCategoryIndex);
			this[this.activePanel].blurPanel();
			this[this.activePanel].stopTimer();
			this.focusSectionsPanel();
		}
	} else if (this.activePanel == "programs") {
		var index = this.pager.paging.index;
		this.pager.prev();
		if (index > 0) {
			$("#pagerInfo").html(this.pager.paging.getPagerInfo());
		}
		$('#scrollArchPlusPrograms').sfScroll('move', this.pager.scroll.pageNumber);
	}
};

/**
 * @event
 * @description Occurs when down button pressed 
 */
SceneArchPlus.handleDown = function () {
	if (this.activePanel == "sectionsPanel") {
		this.sectionsPanel.blurPanel();
		if (this.programs.length > 0) {
			this.focusPrograms();
		} else {
			this.focusActiveCategories();
		}
	} else if (this.activePanel == "popularCategoriesPanel" || this.activePanel == "channelsCategoriesPanel" || this.activePanel == "genresCategoriesPanel") {
		this[this.activePanel].focusNext();
	} else if (this.activePanel == "programs") {
		this.pager.next();
		$("#pagerInfo").html(this.pager.paging.getPagerInfo());
		$('#scrollArchPlusPrograms').sfScroll('move', this.pager.scroll.pageNumber);
	}
};

/**
 * @event
 * @description Occurs when enter button pressed 
 */
SceneArchPlus.handleEnter = function () {
	if (this.activePanel == "genresCategoriesPanel") {
		Logger.write("Current element: " + this.genres.panel.getFocusedId(), "ArchPlus", true);
		var id = parseInt(this.genres.panel.getFocusedId().replace("#divgenreCategory", ""));
		var genres = this.getChildren(this.genres.active, id);
		if (genres) {
			this.bindCategories(this.genres, genres);
			this.genres.history.push(this.genres.panel.getCurrentPosition());
			this.genres.panel.focusPanel();
			this.genres.panel.invokeSelectedChangedCallback();
		}
	} else if (this.activePanel == "channelsCategoriesPanel") {
		var id = this.channels.panel.getFocusedId().replace("#divchannelCategory", "");
		Logger.write("Current element: " + this.channels.panel.getFocusedId() + ", id: " + id, "ArchPlus", true);
		var item = this.getItem(this.channels.active, id);
		if (!item) {
			return;
		}
		
		var categories = item.subCategories;
		if (item.isChannel) {
			SceneArchPlus.currentChannelId = item.id;
			item.updateSubCategoriesParent();
		}
		
		this.bindCategories(this.channels, categories);
		this.channels.history.push(this.channels.panel.getCurrentPosition());
		this.channels.panel.focusPanel();
	} else if (this.activePanel == "programs") {
		var idx = $('#lbArchPlusPrograms').sfList('getIndex');
		var program = SceneArchPlus.programs[idx];
		var hasSeries = program.isContainer && program.episodeNum === 1 || program.episodeNum > 1;
		if (hasSeries && !SceneArchPlus.request.channelId) {
			sf.scene.hide(AppContext.CurrentScene);
			sf.scene.show('ProgramEpisodes');
			sf.scene.focus('ProgramEpisodes');
			SceneProgramEpisodes.loadEpisodes(program.id, "ArchPlus");
		} else {
			SceneArchPlus.openPlayer();
		}
	}
};

/**
 * @event
 * @description Occurs when yellow (C) button pressed 
 */
SceneArchPlus.handleYellow = function () {
	if (this.activePanel == "programs") {
		var itemIndex = $('#lbArchPlusPrograms').sfList('getIndex');
		var item = SceneArchPlus.programs[itemIndex];
		
		ScenePopUp.setWindow("ArchPlus", item);
		sf.scene.show('PopUp');
		sf.scene.focus('PopUp');
	}
};

/**
 * @event
 * @description Occurs when return button pressed 
 */
SceneArchPlus.handleReturn = function () {
	var result = SceneArchPlus.handleLeft();
	Logger.write("Result: " + result, "ArchPlus", true);
	if (!result) {
		SceneArchPlus.backToMainMenu();
	}
};

/**
 * @event
 * @description Occurs when exit button pressed 
 */
SceneArchPlus.handleExit = function () {
	var resultCallback = function (result) {
		if (result) {
			AppEvents.Exit();
		}
	};
	SceneDialogPopUp.showExitDialog("ArchPlus", resultCallback);
};

/**
 * @function
 * @description Provides moving back to main menu 
 */
SceneArchPlus.backToMainMenu = function () {
    try
    {
        sf.scene.hide('ArchPlus');
        SceneMainMenu.show();
    }
    catch (error)
    {
        ErrorHandler.HandleError(error, "ArchPlus");
    }  
};

/**
 * @event
 * @description This function will be called when the scene manager show this scene 
 */
SceneArchPlus.prototype.handleShow = function () {
	alert("SceneArchPlus.handleShow()");
	SceneArchPlus.translate();
};

/**
 * @event
 * @description This function will be called when the scene manager hide this scene 
 */
SceneArchPlus.prototype.handleHide = function () {
	alert("SceneArchPlus.handleHide()");
	SceneArchPlus.popularCategoriesPanel.stopTimer();
	SceneArchPlus.channelsCategoriesPanel.stopTimer();
	SceneArchPlus.genresCategoriesPanel.stopTimer();
	Helpers.hideLoading();
};

/**
 * @event
 * @description This function will be called when the scene manager focus this scene
 */
SceneArchPlus.prototype.handleFocus = function () {
	alert("SceneArchPlus.handleFocus()");
	AppContext.CurrentScene = "ArchPlus";
	if (ScenePlayer.isPlaying == true) {
		Helpers.setBackgroundOpacity();
	}
	SceneArchPlus.setHelpBar();
	if (ErrorHandler.NeedToRefresh[AppContext.CurrentScene]) {
		SceneArchPlus.refresh();
		ErrorHandler.NeedToRefresh[AppContext.CurrentScene] = false;
    } else {
		var focusedSection = SceneArchPlus.sectionsPanel.getFocusedId();
		if (focusedSection.indexOf("popular") > -1) {
			SceneArchPlus.popularCategoriesPanel.invokeSelectedChangedCallback();
		} else if (focusedSection.indexOf("channels") > -1) {
			SceneArchPlus.channelsCategoriesPanel.invokeSelectedChangedCallback();
		} else if (focusedSection.indexOf("genres") > -1) {
			SceneArchPlus.genresCategoriesPanel.invokeSelectedChangedCallback();
		} else if (SceneArchPlus.sceneWasntActive) {
			SceneArchPlus.sceneWasntActive = false;
			SceneArchPlus.bindPrograms();
		}
	}
	
	if (SceneArchPlus.isLoading) {
		Logger.write("Showing loading indicator [handleFocus]", "ArchPlus");
		Helpers.showLoading();
	}
};

/**
 * @event
 * @description This function will be called when the scene manager move focus to another scene from this scene
 */
SceneArchPlus.prototype.handleBlur = function () {
	alert("SceneArchPlus.handleBlur()");
};

/**
 * @event
 * @description This function will be called when some key was pressed on the scene.
 */
SceneArchPlus.prototype.handleKeyDown = function (keyCode) {
	alert("SceneArchPlus.handleKeyDown(" + keyCode + ")");
	// TODO : write an key event handler when this scene get focued
	if (Helpers.handleGlobalKeys(keyCode)) {
		return;
	}
	switch (keyCode) {
		case sf.key.LEFT:
            SceneArchPlus.handleLeft();
			break;
		case sf.key.RIGHT:
            SceneArchPlus.handleRight();
			break;
		case sf.key.UP:
            SceneArchPlus.handleUp();
			break;
		case sf.key.DOWN:
            SceneArchPlus.handleDown();
			break;

		case sf.key.ENTER:
            SceneArchPlus.handleEnter();
			break;
		case sf.key.BLUE:
            if  (ScenePlayer.isPlaying == true) {
				sf.scene.hide('ArchPlus');
				sf.scene.show('Player');
				sf.scene.focus('Player');
            } else {
            	SceneArchPlus.refresh();
            }
            break;
		case sf.key.YELLOW:
            try {
                SceneArchPlus.handleYellow();
            } catch (error) {
				ErrorHandler.HandleError(error, "ArchPlus");
            } finally {
				break;
            }
		case sf.key.GREEN:
            try {
				Logger.write("Showing search window", "ArchPlus");
				var okFunc = function (numCharResult, result) {
					sf.scene.hide('ArchPlus');
					SceneSearchChannels.show();
					SceneSearchChannels.loadResults(result, "ArchPlus");
				};
				SceneTextboxPopUp.setCallbacks(okFunc, "ArchPlus");
				SceneTextboxPopUp.show();
            } catch (error) {
				ErrorHandler.HandleError(error, "ArchPlus");
            }    
            break;
		case sf.key.RED:
            try {
                sf.scene.hide('ArchPlus');
                sf.scene.show('Settings');
                sf.scene.focus('Settings');
                SceneSettings.selectAccount();
            } catch (error) {
                ErrorHandler.HandleError(error, "ArchPlus");  
            }  
            break;
		case sf.key.RETURN:
            widgetAPI.blockNavigation(event);
			SceneArchPlus.handleReturn();
			break;
		case sf.key.EXIT:
			try {
				widgetAPI.blockNavigation(event);
				SceneArchPlus.handleExit();
			} catch (error) {
				ErrorHandler.HandleError(error, "ArchPlus");
			}
			break;
		case sf.key.STOP:
            if  (ScenePlayer.isPlaying == true) {
				ScenePlayer.stop();
				SceneArchPlus.setHelpBar();
            }
            break;
	}
};
