/**
 * @class Describes Archive TV page (scene)
 * @param options - options for page.
 */
function SceneArchTv(options) {
	/**
	 * @field
	 * @description option of the page.
	 */
	this.options = options;
}

/**
 * @function
 * @description this function will be called only once when the scene manager show this scene first time
 * initialize the scene controls and styles, and initialize your variables here 
 * scene HTML and CSS will be loaded before this function is called
 */
SceneArchTv.prototype.initialize = function () {
	Logger.write("SceneArchTv.initialize()", "ArchTv");
	ErrorHandler.NeedToRefresh["ArchTv"] = false;
    SceneArchTv.hideImage();    
    SceneArchTv.currentLang = AppContext.ClientSettings.Language;
    SceneArchTv.initData();
    SceneArchTv.maxName = 52;
    $('#scrollArchTvPrograms').sfScroll({pages:'1'});
    $('#lblArchProgramDescription').sfLabel({text:''});
    $('#imgTvSearch').sfImage({src:'images/Search.png'});    
    $('#lblArchTvTitle').sfLabel({text:Resources.getResources().LabelArchTv.toLowerCase()});
    $('#divTvSearch').sfLabel({text:Resources.getResources().Search});
    SceneArchTv.loadPanels();
    SceneArchTv.setHelpBar();  
};

/**
 * @function
 * @description Provides refreshes data on scene
 */
 SceneArchTv.refresh = function() {
	SceneArchTv.uninitialize();
    SceneArchTv.initData();
	Logger.write("Panels: " + $('#SceneArchTv #panelsArchTv').html(), "ArchTv", true);
    $('#SceneArchTv #panelsArchTv').empty();
	$('#SceneArchTv #panelsArchTv').css("opacity", "0");
	Logger.write("Panels: " + $('#SceneArchTv #panelsArchTv').html(), "ArchTv", true);
    $('#scrollArchTvPrograms').sfScroll({pages:'1'});
	$('#scrollArchTvPrograms').css({'visibility':'hidden'});
    $('#lblArchProgramDescription').sfLabel({text:'', width:'710px'});
    $('#imgTvSearch').sfImage({src:'images/Search.png'});
    SceneArchTv.hideImage();
    $('#lblArchTvTitle').sfLabel({text:Resources.getResources().LabelArchTv.toLowerCase()});
    $('#divTvSearch').sfLabel({text:Resources.getResources().Search});
	$("#lbArchTvPrograms").css({'visibility': 'hidden'});
	$("#panelsArchTv").css({'visibility': 'hidden'});
	$('#lblArchProgramDescription').empty();
	$("#SceneArchTv .programImage").hide();
    SceneArchTv.loadPanels();
 };
 
 /**
  * @function
  * @description Initialize default variables
  */
SceneArchTv.initData = function()
{
	/**
	 * @field
	 * @description 
	 * @default 14
	 */
    this.daysFor = 14;
    /**
	 * @field
	 * @description programs on the one page.
	 * @default 6
	 */
    this.programsPerPage = 6;
    /**
	 * @field
	 * @description panel of the channels.
	 * @default null
	 */
    this.channelsPanel = null;
    /**
	 * @field
	 * @description panel of the dates.
	 * @default null
	 */
    this.datesPanel = null;
};

/**
 * @function
 * @description Provides displaying of the loading element
 */
SceneArchTv.showLoading = function () {
	SceneArchTv.isLoading = true;
	if (AppContext.CurrentScene == "ArchTv") {
		Helpers.showLoading();
	}
};

/**
 * @function
 * @description Provides hiding of the loading element
 */
SceneArchTv.hideLoading = function () {
	SceneArchTv.isLoading = false;
	if (AppContext.CurrentScene == "ArchTv") {
		Helpers.hideLoading();
	}
};

/**
 * @function
 * @description Provides uninitialize of the page (scene).
 */
SceneArchTv.uninitialize = function()
{
	if (this.datesPanel) {
		this.datesPanel.unload();
		this.datesPanel = null;
	}
	if (this.channelsPanel) {
		this.channelsPanel.unload();
		this.channelsPanel = null;
	}
    $('#lbArchTvPrograms').sfList('clear');
    SceneArchTv.activePrograms = null;
    SceneArchTv.imagesUrls = null;
    Logger.write("======== Uninitialized", "ArchTv");
};

/**
 * @function
 * @description Provides loading dates on page.
 * @param {function} callback - occurs when dates are loaded. 
 */
SceneArchTv.loadDates = function (callback) {
	var getTimeCallback = function (result) {
		var tmpDate = result;
		var shift = 0;
		for (var i = 0; i < AppContext.TimeZones.length; i++) {
			if (AppContext.ClientSettings.timeZoneName == AppContext.TimeZones[i].id) {
				shift = parseInt(AppContext.TimeZones[i].utcOffset);
				break;
			}
		}
		tmpDate.setMinutes(tmpDate.getMinutes() + shift);
		var time = Helpers.getUtcTime(tmpDate);
		
		time.setHours(0, 0, 0, 0);
		var elements = [];
		for (var i = 0; i < SceneArchTv.daysFor; i++) {
			var dateId = DateHelper.convertToString(time).replace(/\-/g, "").substr(0, 8);
			elements[i] = new PanelElement(dateId, DateHelper.getArchDate(time));
			time.setDate(time.getDate()-1);
		}

		SceneArchTv.datesPanel = new SlidingPanel(120, 80, elements, "Date", SceneArchTv.selectedChangedCallback, 170, orientations.horizontal, true, null, null, 1000, true, 4, 4);
		$('#SceneArchTv #panelsArchTv').append(SceneArchTv.datesPanel.panel);

		if (!!callback) {
			callback();
		}
	};

	var errCallback = function (jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "LiveTv", true, textStatus, jqXHR);
	};
	
	JsonServices.media.GetCurrentUtcTime(getTimeCallback, errCallback);
};

/**
 * @function
 * @description Provides checking complete of the data load.
 */
SceneArchTv.checkLoadDataCompleted = function () {
	if (!!SceneArchTv.datesPanel && !!SceneArchTv.channelsPanel) {
		SceneArchTv.onLoadPanelsCompleted();
	}
};

/**
 * @event
 * @description Occurs when the panels are loaded
 */
SceneArchTv.onLoadPanelsCompleted = function()
{
	/**
	 * @field
	 * @description flag, that show success of the data loading.
	 */
    this.isDataLoaded = true;
	if (this.channelsPanel.elements.length > 0) {
		Logger.write("[Arch Tv] Data loaded!", "ArchTv");
		$('#SceneArchTv #panelsArchTv').css('visibility', 'visible');
		$('#SceneArchTv #panelsArchTv').css("opacity", "1");
		/**
		 * @field
		 * @description focused panel.
		 */
		this.focusedPanel = "channelsPanel";
		this[this.focusedPanel].focusPanel();
		this.loadPrograms();
		this.showCounter();
	} else {
		$('#SceneArchTv #panelsArchTv').css('visibility', 'hidden');
		$('#SceneArchTv #panelsArchTv').css("opacity", "0");
		$('#lbArchTvPrograms').css({'visibility':'visible'});
		$('#lbArchTvPrograms').empty();
		$("#lbArchTvPrograms").append("<div class='noData'>" + Resources.getResources().NoData + "</div>");
		SceneArchTv.hideLoading();
	}
};

/**
 * @event
 * @description Occurs when selection on this page (scene) is changed
 */
SceneArchTv.selectedChangedCallback = function (elementId) {
    if (typeof(SceneArchTv.channelsPanel) != "undefined" && typeof(SceneArchTv.datesPanel) != "undefined") {
        SceneArchTv.loadPrograms();
    }
};

/**
 * @function
 * @description Provides the hiding of the program image
 */
SceneArchTv.hideImage = function(){
        $("#imgArchProgram").sfImage({src:""});
        $("#SceneArchTv .programImage").hide();
};

/**
 * @function
 * @description Provides programs binding
 */
SceneArchTv.bindPrograms = function () {
	var progs = [];
	for (var i = 0; i < SceneArchTv.activePrograms.length; i++) {
		var time = DateHelper.getTime(DateHelper.parseDate(SceneArchTv.activePrograms[i].Date), false, true);
		var programName = SceneArchTv.activePrograms[i].TvProgramName;
		programName = programName.length > 60 ? programName.substring(0, 57) + "..." : programName;
		progs.push("<span class='programTime'>" + time + "</span><span class='programTitle'>" + Helpers.decodeHtml(programName + "</span>"));
	}
	$('#lbArchTvPrograms').sfList({data: progs, itemsPerPage: SceneArchTv.programsPerPage});
	$('#lbArchTvPrograms').sfList('move', 0);
	$('#lbArchTvPrograms').sfList('blur');
	var page = Math.ceil(progs.length/SceneArchTv.programsPerPage);
	$('#scrollArchTvPrograms').sfScroll({pages: page});
	$('#scrollArchTvPrograms').sfScroll('focus');
	$('#scrollArchTvPrograms').css({'visibility':'visible'});
	$('#lbArchTvPrograms').css({'visibility':'visible'});
	if (progs.length > 0) {
		$('#lbArchTvPrograms').sfList('show');
	}
};

/**
 * @event
 * @description Occurs when programs loaded.
 * @param {object} result - loaded programs.
 */
SceneArchTv.onProgramsLoaded = function (result) {
	Logger.write("SceneArchTv.onProgramsLoaded()", "ArchTv", true);
	Logger.write("Total programmmms: " + result.length, "ArchTv", true);
    //try {
		var programs = result.items.length ? result.items : null;
        if (programs) {
			Logger.write("Total programs: " + programs.length, "ArchTv", true);
			var dateId = SceneArchTv.datesPanel.elements[SceneArchTv.datesPanel.index].replace("#divDate", "");
			var date = new Date(parseInt(dateId.substr(0, 4), 10), parseInt(dateId.substr(4, 2), 10)-1, parseInt(dateId.substr(6, 2), 10));
			var dateLocaleFormat = DateHelper.dateToString(date, "%Y-%m-%d");
			date = null;
			dateId = null;
			var resultPrograms = [];
			var progs = [];
			var j = 0;
			for (var i = 0; i < programs.length; i++) {
				programs[i].recentAirTime.startTime.setHours(programs[i].recentAirTime.startTime.getHours() + SceneArchTv.offset - AppContext.deviceOffset/60);
				var programDate = programs[i].recentAirTime.startTime;
				var watchable = programs[i].recentAirTime.archived;
				if ((DateHelper.dateToString(programDate, "%Y-%m-%d") == dateLocaleFormat) && watchable) {
					var time = DateHelper.getTime(programDate, false, true);
					resultPrograms[j] = programs[i];
					var programName = programs[i].name;
					programName = programName.length > 60 ? programName.substring(0, 57) + "..." : programName;
					progs[j++] = "<span class='programTime'>" + time + "</span><span class='programTitle'>" + Helpers.decodeHtml(programName + "</span>");
				}
				programDate = null;
			}
			dateLocaleFormat = null;

			SceneArchTv.activePrograms = resultPrograms;
			//$("#lbArchTvPrograms").empty();
			$('#lbArchTvPrograms').sfList({data: progs, itemsPerPage: SceneArchTv.programsPerPage});
			$('#lbArchTvPrograms').sfList('move', 0);
			$('#lbArchTvPrograms').sfList('blur');
			var page = Math.ceil(progs.length/SceneArchTv.programsPerPage);
			$('#scrollArchTvPrograms').sfScroll({pages: page});
			$('#scrollArchTvPrograms').sfScroll('focus');
			$('#scrollArchTvPrograms').css({'visibility':'visible'});
			$('#lbArchTvPrograms').css({'visibility':'visible'});
			if (progs.length > 0) {
				$('#lbArchTvPrograms').sfList('show');
			}
			if (AppContext.CurrentScene != "ArchTv") {
				SceneArchTv.sceneWasntActive = true;
			}
        } else {
			SceneArchTv.hideLoading();
        }
    /*} catch (error) {
            ErrorHandler.HandleError(error,"ArchTv");  
    }*/
    SceneArchTv.hideLoading();
    programms = null;
    SceneArchTv.isDataLoaded = true;
};

/**
 * @function
 * @description Getting and load programs data (async)
 */
SceneArchTv.loadPrograms = function()
{
    this.isDataLoaded = false;
    $('#divPrograms').css('visibility', 'hidden');
    SceneArchTv.showLoading();
    channelId = $(this.channelsPanel.getFocused()).attr('id').replace("divChannel", "");
    var dateId = this.datesPanel.elements[this.datesPanel.index].replace("#divDate", "");
    var date = new Date(parseInt(dateId.substr(0, 4), 10), parseInt(dateId.substr(4, 2), 10)-1, parseInt(dateId.substr(6, 2), 10));
	date.setHours(date.getHours() - SceneArchTv.offset + AppContext.deviceOffset/60);
	var dateTill = new Date(date.getTime());
	dateTill.setDate(dateTill.getDate() + 1);
	Logger.write("Selected date: " + date, "ArchTv", true);
    var perPage = this.programsPerPage;

	var errCallback = function(jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "ArchTv", true, textStatus);
	};
	
	JsonServices.content.GetClientProgramGuide(AppContext.user.sid, "DVR", "AllItems", channelId, 
			null, date, dateTill, null, null, null, 50, 1, AppContext.ClientSettings.shiftTimeZoneName, 
			AppContext.ClientSettings.shiftTimeZoneName, SceneArchTv.onProgramsLoaded, errCallback);
};

/**
 * @event
 * @description Occurs when channels are loaded.
 * @param {object} result - channels.
 */
SceneArchTv.onChannelsLoaded = function (result) {
    try {
        var elements = [];
		var channels = result.items;
		SceneArchTv.channels = {};
        for (var i = 0; i < channels.length; i++) {
			if (channels[i].readyForDvr) {
				var channelName = channels[i].name;
				channelName = channelName.length > 27 ? channelName.substring(0,27) + "..." : channelName;
				var id = channels[i].id;
				var imageUrl = AppContext.ImageUrlTemplate.replace("{0}", id).replace("{1}", 10).replace("{2}", 1) + "&width=32";
				elements.push(new PanelElement(id, channelName, imageUrl));
				if (channels[i].arX && channels[i].arY) {
					SceneArchTv.channels[id] = {
						ar: {
							x: channels[i].arX,
							y: channels[i].arY
						}
					};
				}
			}
        }
        SceneArchTv.channelsPanel = new SlidingPanel(80, 80, elements, "Channel", SceneArchTv.selectedChangedCallback, 300, orientations.horizontal, true, null, null, 1000, true, 3, 3);
        $('#SceneArchTv #panelsArchTv').append(SceneArchTv.channelsPanel.panel);
        if (SceneArchTv.datesPanel && SceneArchTv.channelsPanel) {
            SceneArchTv.onLoadPanelsCompleted();
        }
    } catch (error) {
		ErrorHandler.HandleError(error, "ArchTv");
    }  
};

/**
 * @function
 * @description Getting and load data to panels.
 */
SceneArchTv.loadPanels = function()
{
	this.focusedPanel = null;
	$('#SceneArchTv #panelsArchTv').css('visibility', 'hidden');
	SceneArchTv.showLoading();

	var errCallback = function(jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "ArchTv", true, textStatus);  
	};
	this.offset = AppContext.getNaEstUtcOffset();
	
	JsonServices.content.GetClientChannels(AppContext.user.sid, "DVR", 200, 1, SceneArchTv.onChannelsLoaded, errCallback);
	
	//this.datesPanel.hideElems()
	SceneArchTv.loadDates(function () {
		SceneArchTv.checkLoadDataCompleted();
	});
};

/**
 * @function
 * @description Marks movie as bookmarked or not bookmared 
 * @param {number} id - movie's identifier
 * @param {boolean} status - bookmarked or not bookmared.
 */
SceneArchTv.changeFavorite = function (id, status) {
	if (!SceneArchTv.activePrograms || SceneArchTv.activePrograms.length == 0) {
		return;
	}
	
	for (var i = 0; i < SceneArchTv.activePrograms.length; i++) {
		var program = SceneArchTv.activePrograms[i];
		if (program.id == id) {
			program.bookmarked = status;
			break;
		}
	}
};

/**
 * @function
 * @description Gets aspect ration of the channel.
 */
SceneArchTv.getChannelAr = function (channelId) {
	if (!SceneArchTv.channels || !SceneArchTv.channels[channelId]) {
		if (channelId == 1 || channelId == 300473) {
			return {
				x: 16,
				y: 9
			};
		}
		return null;
	}
	
	return SceneArchTv.channels[channelId].ar;
};

/**
 * @function
 * @description Changes focused panel
 * @param {object} newPanel - panel to focus.
 */
SceneArchTv.changePanelFocus = function(newPanel)
{
    this[this.focusedPanel].blurPanel();
    this.focusedPanel = newPanel;
    this[this.focusedPanel].focusPanel();
};

/**
 * @function
 * @description Provides displaying channel counter
 */
SceneArchTv.showCounter = function () {
	Logger.write("SceneArchTv.showCounter()", "ArchTv");
	var index = SceneArchTv.channelsPanel.index + 1;
	var total = SceneArchTv.channelsPanel.elements.length;
	$("#archTvChannelCounter").empty();
	$("#archTvChannelCounter").append(index + " | " + total);
};

/**
 * @event
 * @description Handles left button press
 */
SceneArchTv.handleLeft = function ()
{
    try
    {
		if (this.focusedPanel) {
			if (this.focusedPanel != "programsPanel")
			{
				if (this.isDataLoaded)
				{
					this[this.focusedPanel].focusPrev();
					SceneArchTv.showCounter();
				}
			}
			else
			{
				$('#lbArchTvPrograms').sfList('blur');
				this.focusedPanel = "datesPanel";
				this[this.focusedPanel].focusPanel();
				SceneArchTv.hideImage();
				$('#lblArchProgramDescription').sfLabel({text: ''});
			}
		}
    }
    catch (error)
    {
		ErrorHandler.HandleError(error,"ArchTv");
    }
};

/**
 * @event
 * @description Handles right button press
 */
SceneArchTv.handleRight = function ()
{
    try
    {
		if (this.focusedPanel) {
			if (this.focusedPanel != "programsPanel")
			{
				if (this.isDataLoaded)
				{
					this[this.focusedPanel].focusNext();
					SceneArchTv.showCounter();
				}
			}
			else
			{
				$('#lbArchTvPrograms').sfList('blur');
				this.focusedPanel = "datesPanel";
				this[this.focusedPanel].focusPanel();   
				SceneArchTv.hideImage();
				$('#lblArchProgramDescription').sfLabel({text: ''});
			}
		}
    }
    catch (error)
    {
		ErrorHandler.HandleError(error,"ArchTv");  
    }  
};

/**
 * @event
 * @description Handles down button press
 */
SceneArchTv.handleDown = function()
{
    //$('#scrollArchTvPrograms').sfScroll('next');
    try
    {
        if (this.isDataLoaded)
        {
            if (this.focusedPanel == "channelsPanel")
            {
                this.changePanelFocus("datesPanel");
            }
            else if (this.focusedPanel == "datesPanel" && this.activePrograms.length > 0)
            {
                this[this.focusedPanel].blurPanel();
                this.focusedPanel = "programsPanel";
                $('#lbArchTvPrograms').sfList('focus');
                this.showDescription(this.activePrograms[$('#lbArchTvPrograms').sfList('getIndex')]);
            }
            else if (this.focusedPanel == "programsPanel")
            {
                var idx = $('#lbArchTvPrograms').sfList('getIndex');
                if (((idx+1) % this.programsPerPage) == 0)
                {
                    var moveTo = Math.ceil(idx/this.programsPerPage);
                    $('#scrollArchTvPrograms').sfScroll('move', moveTo);
                }              
              
                if (idx == (this.activePrograms.length - 1))
                {            
					return;
                }    
                
                $('#lbArchTvPrograms').sfList('next');
                this.showDescription(this.activePrograms[$('#lbArchTvPrograms').sfList('getIndex')]);
            }
        }
    }
    catch(error)
    {
        ErrorHandler.HandleError(error,"ArchTv");  
    }  
};

/**
 * @event
 * @description Handles up button press
 */
SceneArchTv.handleUp = function()
{
    try
    {
            if (this.isDataLoaded)
            {
                if (this.focusedPanel == "datesPanel")
                {
                    this.changePanelFocus("channelsPanel");
                }
                else if(this.focusedPanel == "programsPanel")
                {
                    var idx = $('#lbArchTvPrograms').sfList('getIndex');
                    if (idx == 0)
                    {
                        $('#lbArchTvPrograms').sfList('blur');
                        this.focusedPanel = "datesPanel";
                        this[this.focusedPanel].focusPanel();                
                        SceneArchTv.hideImage();
                        $('#lblArchProgramDescription').sfLabel({text: ''});
                    }
                    else
                    {
                        if (((idx+1) % this.programsPerPage) == 1)
                        {
                            var moveTo = Math.ceil(idx/this.programsPerPage)-1;
                            $('#scrollArchTvPrograms').sfScroll('move', moveTo);
                        }
                        $('#lbArchTvPrograms').sfList('prev');
                        this.showDescription(this.activePrograms[$('#lbArchTvPrograms').sfList('getIndex')]);
                    }
                }
            }
    }
    catch(error)
    {
            ErrorHandler.HandleError(error,"ArchTv");  
    }  
};

/**
 * @function
 * @description Shows description for selected program
 * @param {object} programData - data of the program.
 */
SceneArchTv.showDescription = function(programData)
{   
	setTimeout(function () {
		if (typeof(programData) == "undefined" )
			return;

		var description = "";
		if (programData.description) {
			var desc =  Helpers.decodeHtml(Helpers.escapeHTML(programData.description)) ;        
			description += desc;
		}
		description = Helpers.removeQuotes(description);
		$('#lblArchProgramDescription').empty();
		$('#lblArchProgramDescription').append(description);
		if (programData.imageCount > 0) {
			$("#SceneArchTv #lblArchProgramDescription").removeClass("noImage");
			var imageUrl = AppContext.ImageUrlTemplate.replace("{0}", programData.id).replace("{1}", 28).replace("{2}", 1) + "&width=152";
			$("#imgArchProgram").sfImage({src: imageUrl});
			$("#SceneArchTv .programImage").show();
		} else {
			$("#SceneArchTv .programImage").hide();
			$("#SceneArchTv #lblArchProgramDescription").addClass("noImage");
		}
	}, 1);
};

/**
 * @function
 * @description Plays selected program
 */
SceneArchTv.play = function () {
    var idx = $('#lbArchTvPrograms').sfList('getIndex');      
    if (ScenePlayer.isPlaying && ScenePlayer.contentType == "DVR" && ScenePlayer.archProgramPicker) {
		if (ScenePlayer.archProgramPicker.index == idx) {
			sf.scene.hide(AppContext.CurrentScene);
			sf.scene.show('Player');
			sf.scene.focus('Player');
			return;
		}
	}
    SceneArchTv.playProgram(idx);
};
   
/**
 * @function
 * @description Plays specified program 
 * @param {number} programIndex - program id.
 */
SceneArchTv.playProgram = function (programIndex) {
	var program = SceneArchTv.activePrograms[programIndex];
	
	var callback = function (response) {
		try {
			ScenePlayer.archProgramPicker = new ArchProgramPicker(SceneArchTv.activePrograms, SceneArchTv.playProgram, programIndex);
			$('#lbArchTvPrograms').sfList('move', programIndex);
			SceneArchTv.showDescription(SceneArchTv.activePrograms[$('#lbArchTvPrograms').sfList('getIndex')]);
			var tmpPrograms = SceneArchTv.activePrograms.slice(programIndex);
			ASXLoader.loadProgramsDuration(tmpPrograms);
			Logger.write("Program duration: " + program.Duration, "ArchTv", true);
			sf.scene.hide(AppContext.CurrentScene);
			sf.scene.show('Player');
			var video = program;
			video.name = program.name;
			video.currentTime = 0;
			video.cbPlay = SceneArchTv.playProgram;
			video.duration = program.length;
			video.url = response.URL;
			video.ar = Helpers.parseAr(response.DAR);
			video.playlist = response.Playlist;
			video.contentType = "DVR";
			ScenePlayer.setContent(video, "ArchTv");
			sf.scene.focus('Player');
		} catch (error) {
			ErrorHandler.HandleError(error, "ArchTv");  
		}
	};
	
	var errCallback = function (jqXHR, textStatus, error) {
		ErrorHandler.HandleError(error, "ArchTv", true, textStatus);
	};
	
	try {
		JsonServices.media.GetClientStreamUri(
			AppContext.user.sid, "DVR", program.id,
			AppContext.ClientSettings.bAId, AppContext.ClientSettings.cdnId,
			AppContext.ClientSettings.qualityPreset, AppContext.ClientSettings.shiftTimeZoneName,
			callback, errCallback
		);
	} catch (error) {
		ErrorHandler.HandleError(error,"ArchTv");
	}
};

/**
 * @event
 * @description Handles enter button press
 */
SceneArchTv.handleEnter = function()
{
	try
	{
		if (this.focusedPanel == "programsPanel")
		{
			SceneArchTv.play();
		}
	}
	catch (error)
	{
		ErrorHandler.HandleError(error,"ArchTv");
	}
};

/**
 * @event
 * @description Handles yellow (C) button press
 */
SceneArchTv.handleYellow = function ()
{
    try
    {
		if (this.focusedPanel == "programsPanel")
		{
			var idx = $('#lbArchTvPrograms').sfList('getIndex');
			var actProgram = this.activePrograms[idx];
			ScenePopUp.setWindow("ArchTv", actProgram);
			sf.scene.show('PopUp');
			sf.scene.focus('PopUp'); 
		}
    }
    catch(error)
    {
		ErrorHandler.HandleError(error, "ArchTv");
    }
};

/**
 * @event
 * @description Handles numeric button press
 * @param {number} number - number of the pressed button
 */
SceneArchTv.handleNum = function (number) {
	Logger.write("handling num: " + number, "ArchTv", false);
	var num = parseInt(this.newChannel + number, 10);
	if (num <= this.channelsPanel.getTotal()) {
		this.newChannel += number;
	
		if (SceneArchTv.channelTimeout) {
			clearTimeout(SceneArchTv.channelTimeout);
		}
		
		var total = SceneArchTv.channelsPanel.elements.length;
		$("#archTvChannelCounter").empty();
		$("#archTvChannelCounter").append(this.newChannel + "- | " + total);

		var callback = function () {
			var num = parseInt(SceneArchTv.newChannel);
			SceneArchTv.channelsPanel.setIndex(num - 1);
			SceneArchTv.showCounter();
			SceneArchTv.newChannel = "";
		};
		SceneArchTv.channelTimeout = setTimeout(callback, 2000);
	}
};

/**
 * @function
 * @description Moves focus back to MainMenu scene
 */
SceneArchTv.backToMainMenu = function ()
{
    try
    {
		sf.scene.hide('ArchTv');
		SceneMainMenu.show();
     }
    catch (error)
    {
		ErrorHandler.HandleError(error,"ArchTv");
    }  
};

/**
 * @event
 * @description Occurs when exit button was pressed.
 */
SceneArchTv.handleExit = function () {
	var resultCallback = function (result) {
		if (result) {
			AppEvents.Exit();
		}
	};
	SceneDialogPopUp.showExitDialog("ArchTv", resultCallback);
};

/**
 * @function
 * @description Sets help bar content.
 */
SceneArchTv.setHelpBar = function () {
    if (ScenePlayer.isPlaying)
    {
		$('#keysArchTv').sfKeyHelp({
			'number': Resources.getResources().ChangeChannel,
			'red':Resources.getResources().LabelSettings, 
			'green':Resources.getResources().Search,
			'yellow':Resources.getResources().Info, 
			'blue':Resources.getResources().Player, 
			'stop':Resources.getResources().Stop,
			'updown':'', 'leftright':Resources.getResources().MoveCursor,
			'enter':Resources.getResources().Select, 
			'return':Resources.getResources().BackToMainMenu
		});
     }
     else
     {
		$('#keysArchTv').sfKeyHelp({
			'number': Resources.getResources().ChangeChannel,
			'red':Resources.getResources().LabelSettings,
			'green':Resources.getResources().Search, 
			'yellow':Resources.getResources().Info,
			'blue':Resources.getResources().Refresh,
			'updown':'', 'leftright':Resources.getResources().MoveCursor,
			'enter':Resources.getResources().Select,
			'return':Resources.getResources().BackToMainMenu
		});
     }
};

/**
 * @event
 * @description This function will be called when the scene manager show this scene 
 */
SceneArchTv.prototype.handleShow = function () {
	Logger.write("SceneArchTv.handleShow()", "ArchTv");
    if(SceneArchTv.currentLang != AppContext.ClientSettings.Language) {
            $('#lblArchTvTitle').sfLabel({text:Resources.getResources().LabelArchTv.toLowerCase()});
            $('#divTvSearch').sfLabel({text:Resources.getResources().Search});
            SceneArchTv.setHelpBar();
            SceneArchTv.loadDates(function () {
				SceneArchTv.currentLang = AppContext.ClientSettings.Language;
				SceneArchTv.onLoadPanelsCompleted();
			});
    }
    
    SceneArchTv.newChannel = "";
};

/**
 * @event
 * @description This function will be called when the scene manager hide this scene  
 */
SceneArchTv.prototype.handleHide = function () {
	Logger.write("SceneArchTv.handleHide()", "ArchTv");
	Helpers.hideLoading();
};

/**
 * @event
 * @description This function will be called when the scene manager focus this scene
 */
SceneArchTv.prototype.handleFocus = function () {
    Logger.write("SceneArchTv.handleFocus()", "ArchTv");
	AppContext.CurrentScene = "ArchTv";
	if  (ScenePlayer.isPlaying == true) {
		Helpers.setBackgroundOpacity();
	}
    if (ErrorHandler.NeedToRefresh[AppContext.CurrentScene]) {
		SceneArchTv.refresh();
		ErrorHandler.NeedToRefresh[AppContext.CurrentScene] = false;
    } else if (SceneArchTv.isLoading) {
		Helpers.showLoading();
	} else if (SceneArchTv.sceneWasntActive) {
		SceneArchTv.sceneWasntActive = false;
		SceneArchTv.bindPrograms();
	}
    SceneArchTv.setHelpBar();
};

/**
 * @event
 * @description This function will be called when the scene manager move focus to another scene from this scene
 */
SceneArchTv.prototype.handleBlur = function () {
	Logger.write("SceneArchTv.handleBlur()", "ArchTv");
    //SceneArchTv.uninitialize();
};

/**
 * @event
 * @description This function will be called when some key was pressed on the scene.
 */
SceneArchTv.prototype.handleKeyDown = function (keyCode) {
	Logger.write("SceneArchTv.handleKeyDown(" + keyCode + ")", "ArchTv");
	// TODO : write an key event handler when this scene get focued
	if (Helpers.handleGlobalKeys(keyCode)) {
		return;
	}
	switch (keyCode) {
		case sf.key.N0:
			if (SceneArchTv.newChannel) {
				SceneArchTv.handleNum("0");
			}
			break;
		case sf.key.N1:
			SceneArchTv.handleNum("1");
			break;
		case sf.key.N2:
			SceneArchTv.handleNum("2");
			break;
		case sf.key.N3:
			SceneArchTv.handleNum("3");
			break;
		case sf.key.N4:
			SceneArchTv.handleNum("4");
			break;
		case sf.key.N5:
			SceneArchTv.handleNum("5");
			break;
		case sf.key.N6:
			SceneArchTv.handleNum("6");
			break;
		case sf.key.N7:
			SceneArchTv.handleNum("7");
			break;
		case sf.key.N8:
			SceneArchTv.handleNum("8");
			break;
		case sf.key.N9:
			SceneArchTv.handleNum("9");
			break;
		case sf.key.LEFT:
            SceneArchTv.handleLeft();
			break;
		case sf.key.RIGHT:
            SceneArchTv.handleRight();
			break;
		case sf.key.UP:
            SceneArchTv.handleUp();
			break;
		case sf.key.DOWN:
            SceneArchTv.handleDown();
			break;
		case sf.key.ENTER:
            SceneArchTv.handleEnter();
			break;            
        case sf.key.RETURN:            
            widgetAPI.blockNavigation(event);
            SceneArchTv.backToMainMenu();
            break;
        case sf.key.STOP:
            if  (ScenePlayer.isPlaying == true)
            {
                    ScenePlayer.stop();
                    SceneArchTv.setHelpBar();
            }
            break;		
        case sf.key.BLUE:
            if  (ScenePlayer.isPlaying == true) {
	            sf.scene.hide('ArchTv');
	            sf.scene.show('Player');
	            sf.scene.focus('Player');
            } else {
            	SceneArchTv.refresh();
            }
            break;
        case sf.key.YELLOW:
            try
            {
                SceneArchTv.handleYellow();
            }
            catch(error)
            {
				ErrorHandler.HandleError(error,"ArchTv");
            }    
            finally
            {
				break;
            }
        case sf.key.GREEN:
            try
            {
                var okFunc = function(numCharResult, result)
                {
					sf.scene.hide('ArchTv');
                    SceneSearchChannels.show();
					SceneSearchChannels.loadResults(result, "ArchTv");
                };
                SceneTextboxPopUp.setCallbacks(okFunc, "ArchTv");
                SceneTextboxPopUp.show();
            }
            catch(error)
            {
                 ErrorHandler.HandleError(error,"ArchTv");
            }    
            finally
            {
				break;
            }
		case sf.key.RED:
            try
            {
                sf.scene.hide('ArchTv');
                sf.scene.show('Settings');
                sf.scene.focus('Settings');
                SceneSettings.selectAccount();
            }
            catch(error)
            {
                ErrorHandler.HandleError(error,"ArchTv");  
            }  
            break;		
        case sf.key.EXIT:
            try
            {
                widgetAPI.blockNavigation(event);
				SceneArchTv.handleExit();
            }
            catch(error)
            {
                ErrorHandler.HandleError(error,"ArchTv");  
            }  
            break;
         default:         
            widgetAPI.blockNavigation(event);
            break;
	}
};
