/**
 * @class Describes Update manual page (scene)
 * @param options - options of the page
 * @returns
 */
function SceneUpdateManual(options) {
	/**
	 * @field
	 * @description option of the page.
	 */
	this.options = options;
}

/**
 * @function
 * @description this function will be called only once when the scene manager show this scene first time
 * initialize the scene controls and styles, and initialize your variables here 
 * scene HTML and CSS will be loaded before this function is called
 */
SceneUpdateManual.prototype.initialize = function () {
	alert("SceneUpdateManual.initialize()");
};

/**
 * @function 
 * @description Provides translating page (scene) on the language that specified in settings.
 */
SceneUpdateManual.translate = function () {
	if (SceneUpdateManual.currentLang != AppContext.ClientSettings.Language) {
        SceneUpdateManual.currentLang = AppContext.ClientSettings.Language;
        $("#manualTitle .text").sfLabel({text: Resources.getResources().UpdateManualTitle});
    }
};

/**
 * @function
 * @description Sets help bar content.
 */
SceneUpdateManual.setHelpBar = function () {
	$('#keysUpdateManual').sfKeyHelp({
		'return': Resources.getResources().BackToMainMenu
	});
};

/**
 * @function
 * @description Provides showing this page.
 * @param {string} manual - text of the manual for updating
 * @param {string} sender - sender page.
 */
SceneUpdateManual.show = function (manual, sender) {
	if (!manual) {
		return;
	}
	/**
	 * @field
	 * @description sender page.
	 */
	SceneUpdateManual.sender = sender;
	/**
	 * @field
	 * @description manual content
	 */
	SceneUpdateManual.manual = manual;
	
	Logger.write("Manual: " + manual, "UpdateManual", true);
	//$("#manualContent p").html(manual);
	sf.scene.hide(sender);
	sf.scene.show("UpdateManual");
	sf.scene.focus("UpdateManual");
};

/**
 * @function
 * @description Provides moving back to sender page.
 */
SceneUpdateManual.back = function () {
	sf.scene.hide("UpdateManual");
	sf.scene.show(SceneUpdateManual.sender);
	sf.scene.focus(SceneUpdateManual.sender);
	SceneSettings.showUpdates();
};

/**
 * @event
 * @description This function will be called when the scene manager show this scene 
 */
SceneUpdateManual.prototype.handleShow = function () {
	alert("SceneUpdateManual.handleShow()");
};

/**
 * @event
 * @description This function will be called when the scene manager hide this scene  
 */
SceneUpdateManual.prototype.handleHide = function () {
	alert("SceneUpdateManual.handleHide()");
};

/**
 * @event
 * @description This function will be called when the scene manager focus this scene
 */
SceneUpdateManual.prototype.handleFocus = function () {
	alert("SceneUpdateManual.handleFocus()");
	// this 
	SceneUpdateManual.translate();
	SceneUpdateManual.setHelpBar();
	$("#manualContent p").html(SceneUpdateManual.manual[AppContext.ClientSettings.Language.cultureInfo]);
};

/**
 * @event
 * @description This function will be called when the scene manager move focus to another scene from this scene
 */
SceneUpdateManual.prototype.handleBlur = function () {
	alert("SceneUpdateManual.handleBlur()");
};

/**
 * @event
 * @description This function will be called when some key was pressed on the scene.
 */
SceneUpdateManual.prototype.handleKeyDown = function (keyCode) {
	alert("SceneUpdateManual.handleKeyDown(" + keyCode + ")");
	// TODO : write an key event handler when this scene get focused
	switch (keyCode) {
		case sf.key.LEFT:
			break;
		case sf.key.RIGHT:
			break;
		case sf.key.UP:
			break;
		case sf.key.DOWN:
			break;
		case sf.key.RETURN:
			widgetAPI.blockNavigation(event);
			SceneUpdateManual.back();
			break;

		case sf.key.ENTER:
			break;
	}
};
