/**
 * @class Class of service wrappers.
 */

var JsonServices = {
		
		/**
		 * @field 
		 * @description Default service name
		 * @default "service.asmx"
		 */
    defaultName: "service.asmx",

    /**
	 * @class IVS Client service
	 * @description Web and JSON service used for establishing client communication 
	 * with IVSMedia IPTV Distribution Service.
	 */
    client: {
    	/**
    	 * @field 
    	 * @description url of the service
    	 */
        ns: "http://ivsmedia.iptv-distribution.net",
        /**
    	 * @field 
    	 * @description name of the service
    	 */
        serviceName: "JsonClientService.svc/json"
    },

    /**
	 * @class IVS Content service
	 * @description Web and JSON service used for operating media content such as EPG 
	 * (Electronic Program Guide), VOD (Video On Demand) etc that is provided by 
	 * IVSMedia IPTV Distribution Service.
	 */
    content: {
    	/**
    	 * @field 
    	 * @description url of the service
    	 */
        ns: "http://ivsmedia.iptv-distribution.net",
        /**
    	 * @field 
    	 * @description name of the service
    	 */
        serviceName: "JsonContentService.svc/json"
    },
    
    /**
	 * @class IVS Media service
	 * @description MediaService exposes methods to work with images and streams.
	 */
    media: {
    	/**
    	 * @field 
    	 * @description url of the service
    	 */
        ns: "http://ivsmedia.iptv-distribution.net",
        /**
    	 * @field 
    	 * @description name of the service
    	 */
        serviceName: "JsonMediaService.svc/json"
    },
	
    /**
	 * @class
	 * @description Describes parameters for request to service.
	 */
	requests: { },
	
	/**
	 * @class [ENUM] Predefined list of possible subscription services
	 */
	ContentType:  {
		/**
		 * [READONLY]
		 * @description Audio books. Not supported at the moment.
		 * @default 0
		 */
		AudioBooks: 0,
		/**
		 * [READONLY]
		 * @description Archived up to 14 days back live content.
		 * @default 1
		 */
		DVR: 1,
		/**
		 * [READONLY]
		 * @description Games. Not supported at the moment.
		 * @default 2
		 */
		Games: 2,
		/**
		 * [READONLY]
		 * @description Karaoke. Not supported at the moment.
		 * @default 3
		 */
		Karaoke: 3,
		/**
		 * [READONLY]
		 * @description Live or shifted program broadcast.
		 * @default 4
		 */
		LiveTV: 4,
		/**
		 * [READONLY]
		 * @description Music. Not supported at the moment.
		 * @default 5
		 */
		Music: 5,
		/**
		 * [READONLY]
		 * @description Podcast. Not supported at the moment.
		 * @default 6
		 */
		Podcast: 6,
		/**
		 * [READONLY]
		 * @description Radio.
		 * @default 7
		 */
		Radio: 7,
		/**
		 * [READONLY]
		 * @description Archived program broadcasts (a.k.a. Teleteka).
		 * @default 8
		 */
		ArcPlus: 8,
		/**
		 * [READONLY]
		 * @description Movies on demand (a.k.a. Videoteka).
		 * @default 9
		 */
		VOD: 9
	},
	
	/**
	 * @class [ENUM] Definition of types of genres depending on the type of 
	 * client services (such as Live TV, VOD etc.)
	 */
	ContentGenreType: {
		/**
		 * [READONLY]
		 * @description Any new genre type or an error
		 * @default 0
		 */
		UnknownGenreType: 0,
		/**
		 * [READONLY]
		 * @description Type of genres for LiveTV service
		 * @default 1
		 */
		TVGenreType: 1,
		/**
		 * [READONLY]
		 * @description Type of genres for VOD service
		 * @default 2
		 */
		VodGenreType: 2,
		/**
		 * [READONLY]
		 * @description Type of genres for DVR service
		 * @default 3
		 */
		ProgramGenreType: 3,
		/**
		 * [READONLY]
		 * @description Type of genres for ArcPlus service
		 * @default 4
		 */
		ArcplusGenreType: 4
	},
	
	/**
	 * @class [ENUM] Predefined scope of search of on demand items.
	 */	
	OnDemandRequestType: {
		/**
		 * [READONLY]
		 * @description Show all
		 * @default -1
		 */
		AllItems: -1,
		/**
		 * [READONLY]
		 * @description Items added today
		 * @default 0
		 */
		Today: 0,
		/**
		 * [READONLY]
		 * @description Items added yesterday
		 * @default 1
		 */
		Yesterday: 1,
		/**
		 * [READONLY]
		 * @description Items added before yestertoday
		 * @default 2
		 */
		BeforeYesterday: 2,
		/**
		 * [READONLY]
		 * @description Items added 3 days ago
		 * @default 3
		 */
		ThreeDaysAgo: 3,
		/**
		 * [READONLY]
		 * @description Items added 4 days ago
		 * @default 4
		 */
		FourDaysAgo: 4,
		/**
		 * [READONLY]
		 * @description Items added 5 days ago
		 * @default 5
		 */
		FiveDaysAgo: 5,
		/**
		 * [READONLY]
		 * @description Items added 6 days ago
		 * @default 6
		 */
		SixDaysAgo: 6,
		/**
		 * [READONLY]
		 * @description Items added 7 days ago
		 * @default 7
		 */
		SevenDaysAgo: 7,
		/**
		 * [READONLY]
		 * @description Items added last week
		 * @default 100
		 */
		LastWeek: 100,
		/**
		 * [READONLY]
		 * @description Items just added ("postuplenija")
		 * @default 101
		 */
		JustAdded: 101,
		/**
		 * [READONLY]
		 * @description Featured items (Last 20 items marked as featured)
		 * @default 102
		 */
		Featured: 102,
		/**
		 * [READONLY]
		 * @description 100 mostly watched items for last year (365 days)
		 * @default 103
		 */
		TopHundredWatched: 103,
		/**
		 * [READONLY]
		 * @description 100 top rated items
		 * @default 104
		 */
		TopHundredRated: 104,
		/**
		 * [READONLY]
		 * @description 100 most watched items for last month (31 days)
		 * @default 105
		 */
		TopHundredWatchedMonth: 105,
		/**
		 * [READONLY]
		 * @description 100 most watched items for last week (7 days)
		 * @default 106
		 */
		TopHundredWatchedWeek: 106,
		/**
		 * [READONLY]
		 * @description 100 most watched new items (basically it's "new arrivals"/"novinki")
		 * @default 107
		 */
		TopHundredNew: 107,
		/**
		 * [READONLY]
		 * @description Search among actors
		 * @default 120
		 */
		SearchByActors: 120,
		/**
		 * [READONLY]
		 * @description Search among film directors
		 * @default 121
		 */
		SearchByDirectors: 121,
		/**
		 * [READONLY]
		 * @description Search in film names
		 * @default 122
		 */
		SearchByName: 122,
		/**
		 * [READONLY]
		 * @description Search in film descriptions
		 * @default 123
		 */
		SearchByDescription: 123,
		/**
		 * [READONLY]
		 * @description Search by date
		 * @default 124
		 */
		SearchByDate: 124,
		/**
		 * [READONLY]
		 * @description Search by date range
		 * @default 125
		 */
		SearchByDateRange: 125,
		/**
		 * [READONLY]
		 * @description Select free movies/shows from VOD
		 * @default 126
		 */
		FreeItems: 126,
		/**
		 * [READONLY]
		 * @description Extended search by date range
		 * @default 127
		 */
		SearchByDateRangExpanded: 127,
		/**
		 * [READONLY]
		 * @description Search mode that uses keyWord only. For Arcplus (Teleteka) AND DVR 
		 * usage only. The result would be a list of most recent aired programs found by 
		 * keyWord and sorted by Studio/Channel ID.
		 * @default 128
		 */
		SearchByKeywordInChannels: 128
	},
	
	/**
	 * @class [ENUM] Predefined list of quality of the media stream.
	 */	
	StreamQualityPreset: {
		/**
		 * [READONLY]
		 * @description Stands for Variable Quality.<br>
		 * Adaptive Apple or SmoothStreaming depending on application.
		 * @default 0
		 */
		VQ: 0,
		/**
		 * [READONLY]
		 * @description Stands for Standard Quality.<br>
		 * WMV VC1 800 kbs.
		 * @default 1
		 */
		SQ: 1,
		/**
		 * [READONLY]
		 * @description Stands for High Quality.<br>
		 * HWMV VC1 1440 kbs.
		 * @default 2
		 */
		HQ: 2,
		/**
		 * [READONLY]
		 * @description Stands for Great Quality.<br>
		 * Not used yet.
		 * @default 3
		 */
		GQ: 3,
		/**
		 * [READONLY]
		 * @description Stands for Excellent Quality.<br>
		 * Not used yet.
		 * @default 4
		 */
		EQ: 4,
		/**
		 * [READONLY]
		 * @description Stands for Brilliant Quality.<br>
		 * Not used yet.
		 * @default 5
		 */
		BQ: 5
	},
	
	/**
	 * @class [ENUM] Predefined list of types of different items that might be related to another item.
	 */
	RelatedType: {
		/**
		 * [READONLY]
		 * @description For multipart items request the list of other 
		 * parts of the same series. For example, for a sitcom episode, 
		 * requests the list of other episodes of the same season.
		 * @default 1
		 */
		children: 1,
		/**
		 * [READONLY]
		 * @description Request information on the air time of a program. 
		 * For use with Program Guide only.
		 * @default 2
		 */
		showing: 2,
		/**
		 * [READONLY]
		 * @description Find other items of the same genre. 
		 * Returned items change between method calls.
		 * @default 3
		 */
		similar: 3,
		/**
		 * [READONLY]
		 * @description Reserved for future use. Currently has the same 
		 * effect as "children"
		 * @default 4
		 */
		details: 4,
		/**
		 * [READONLY]
		 * @description Request only IDs of child items of a multipart item. 
		 * The type is similar to "children". The difference is that search 
		 * returns only IDs and no other data. This prevents data overhead if it 
		 * is necessary to only form a list of links to child items.
		 * @default 5
		 */
		IDs: 5
	},
	
	/**
	 * @class [ENUM] Predefined list of possible subscription service statuses.
	 */
	Status: {
		/**
		 * [READONLY]
		 * @description Service is not accessible, but available for subscription.
		 * @default 1
		 */
		Available: 1,
		/**
		 * [READONLY]
		 * @description Service is available and accessible.
		 * @default 2
		 */
		Enabled: 2,
		/**
		 * [READONLY]
		 * @description Service is unavailable and shouldn't be shown to clients.
		 * @default 3
		 */
		NA: 3,
		/**
		 * [READONLY]
		 * @description Service is accessible in demo mode only and available for subscription.
		 * @default 4
		 */
		PreviewAvailable: 4
	},
	
	/**
	 * @class [ENUM] Predefined list of registration type (Activate, Confirm for ex.).
	 */
	RegisterInfoType: {
		/**
		 * [READONLY]
		 * @default 0
		 */
		NA: 0,
		/**
		 * [READONLY]
		 * @default 1
		 */
		Activate: 1,
		/**
		 * [READONLY]
		 * @default 2
		 */
		Confirm: 2,
		/**
		 * [READONLY]
		 * @default 3
		 */
		Register: 3
	},
	
	/**
	 * @class [ENUM] Predefined list of possible content advisory raitings.
	 */
	Advisory: {
		/**
		 * [READONLY]
		 * @description Not Rated
		 * @default 0
		 */
		NR: 0,
		/**
		 * [READONLY]
		 * @description Rate 1
		 * @default 0
		 */
		PG1: 1,
		/**
		 * [READONLY]
		 * @description Rate 2
		 * @default 0
		 */
		PG2: 2,
		/**
		 * [READONLY]
		 * @description Rate 3
		 * @default 0
		 */
		PG3: 3,
		/**
		 * [READONLY]
		 * @description Rate 4
		 * @default 0
		 */
		PG4: 4,
		/**
		 * [READONLY]
		 * @description Restricted content
		 * @default 0
		 */
		R: 5
	}
};

/**
 * @field {Array}
 * @description Mapper for converting genre type to content type.
 */
JsonServices.contentTypeGenreTypeMap = { };
JsonServices.contentTypeGenreTypeMap[JsonServices.ContentType.DVR] = JsonServices.ContentGenreType.ProgramGenreType;
JsonServices.contentTypeGenreTypeMap[JsonServices.ContentType.VOD] = JsonServices.ContentGenreType.VodGenreType;
JsonServices.contentTypeGenreTypeMap[JsonServices.ContentType.ArcPlus] = JsonServices.ContentGenreType.ArcplusGenreType;

/**
 * @function
 * @description Invoke-method, which send request to the service.
 * @param data - data which should be sent.
 * @param service - service, where data should be sent.
 * @param method - name of the service function.
 * @param callback - event that occurs when all are done.
 * @param errCallback - event that occurs when error is handle.
 */
JsonServices.invoke = function (data, service, method, callback, errCallback) {
	var url = service.ns + "/" + service.serviceName + "/" + method;
	Logger.write("URL: " +url, "JsonServices", true);
	var json = JSON2.stringify(data);
	var success = function (result) {
		Logger.write("result received: " + result, "JsonServices", true);
		var res;
		if (!result || typeof result !== "string" || result.length == 0) {
			res = null;
		} else {
			res = JSON2.parse(result);
		}
		callback(res);
	};
	Logger.write("Data: " + json, "JsonServices", true);
	$.ajax({
		url: url,
		data: json,
		type: 'POST',
		processData: false,
		contentType: "application/json",
		dataType: 'text',
		success: success,
		error: errCallback,
		statusCode: {
			200: function () {
				Logger.write("Status code: 200", "JsonServices", true);
			}
		}
	});
};

/**
 * @function
 * @description Terminates client session.
 * @param {string} sid - Client's session ID received from {@link Login} method.
 * @param {string} appName - The name of the application which is provided for application developer by 
 * IPTV-DISTRIBUTION This name is used as a part of login process for application identification.
 * @param {string} duid - Hardware identifier of an appliance such as STB, smart TV etc.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.ClientLogout = function (sid, appName, duid, callback, errCallback) {
	var data = {
		cc: {
			appSettings: JsonServices.requests.AppSettings(appName),
			clientCredentials: JsonServices.requests.AccessCredentials(null, null, duid, sid)
		}
	};
	JsonServices.invoke(data, this, "ClientLogout", callback, errCallback);
};

/**
 * @function
 * @description Retrieves a list of messages sent by ISV Media Services to the client
 * @param {string} sid - Logged in client session ID.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.GetMessages = function (sid, callback, errCallback) {
	var data = {
		sessionID: sid
	};
	JsonServices.invoke(data, this, "GetMessages", callback, errCallback);
};

/**
 * @function
 * @description Retrieves client settings based on provided sessionID.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.GetSettings = function (sid, callback, errCallback) {
	Logger.write("JsonServices.client.GetSettings(" + sid + ")", "JsonServices", true);
	var data = {
		sessionID: sid
	};
	JsonServices.invoke(data, this, "GetSettings", callback, errCallback);
};

/**
 * @function
 * @description Method logins client into the system.
 * @param {string} appName - The name of the application which is provided for application developer by 
 * IPTV-DISTRIBUTION This name is used as a part of login process for application identification.
 * @param {string} duid - Hardware identifier of an appliance such as STB, smart TV etc.
 * @param {string} login - End client name for authentication.
 * @param {string} password - End client password for authentication.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.Login = function (appName, duid, login, password, callback, errCallback) {
	Logger.write("JsonServices.client.Login(" + duid + ", " + login + ", " + password + ")", "JsonServices", true);
	var data = {
		cc: {
			appSettings: JsonServices.requests.AppSettings(appName),
			clientCredentials: JsonServices.requests.AccessCredentials(login, password, duid)
		}
	};
	JsonServices.invoke(data, this, "Login", callback, errCallback);
};

/**
 * @function
 * @description Retrieves client settings based on provided sessionID.
 * @param {number} registerType - {@link JsonServices.RegisterInfoType} 
 * @param {string} appName - The name of the application which is provided for application developer by 
 * IPTV-DISTRIBUTION This name is used as a part of login process for application identification.
 * @param {string} email - email address.
 * @param {string} phoneNumber - phone number.
 * @param {string} duid - Hardware identifier of an appliance such as STB, smart TV etc.
 * @param {string} pin - pin code.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.RegisterClient = function (registerType, appName, email, phoneNumber, duid, pin, callback, errCallback) {
	if (typeof(registerType) == "string") {
		registerType = JsonServices.RegisterInfoType[registerType];
	}
	var data = {
		cc: {
			appSettings: JsonServices.requests.AppSettings(appName),
			clientCredentials: JsonServices.requests.AccessCredentials(null, null, duid, null)
		},
		info: JsonServices.requests.RegisterInfo(registerType, email, pin, phoneNumber)
	};
	JsonServices.invoke(data, this, "RegisterClient", callback, errCallback);
};

/**
 * @function
 * @description Informs ISV Media Services which messages were read by the client.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {Array} messageIds - List of message identifiers.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.SetMessagesRead = function (sid, messageIds, callback, errCallback) {
	var data = {
		sessionID: sid,
		messageIDs: messageIds
	};
	JsonServices.invoke(data, this, "SetMessagesRead", callback, errCallback);
};

/**
 * @function
 * @description Updates client settings based on provided sessionID.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {string} language - Client's user interface language preference.
 * @param {string} timeZoneName - Time zone for which TV Guide shows programs time table. 
 * Normally it is the time zone where client watches TV.
 * @param {string} shiftTimeZoneName - Broadcast shift time zone.
 * @param {number} qualityPreset - Media stream quality setting ({@link JsonServices.StreamQualityPreset}).
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.client.SetSettings = function (sid, language, timeZoneName, shiftTimeZoneName, qualityPreset, callback, errCallback) {
	if (typeof(qualityPreset) === "string") {
		qualityPreset = JsonServices.StreamQualityPreset[qualityPreset];
	}
	var data = {
		sessionID: sid,
		cs: JsonServices.requests.ClientSettings(language, timeZoneName, shiftTimeZoneName, qualityPreset)
	};
	JsonServices.invoke(data, this, "SetSettings", callback, errCallback);
};

/**
 * @function
 * @description Adds specified items to the client's favorites collection.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Type of the item's content ({@link JsonServices.ContentType})
 * @param {number} ids - Item unique id.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.AddFavorites = function (sid, contentType, ids, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	var items = ids.map(function (id) {
		return {
			id: id,
			contentType: contentType
		};
	});
	var data = {
		sessionID: sid,
		items: items
	};
	JsonServices.invoke(data, this, "AddFavorites", callback, errCallback);
};

/**
 * @function
 * @description Removes specified items from the client's favorites collection.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Type of the item's content ({@link JsonServices.ContentType})
 * @param {number} ids - Item unique id.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.DeleteFavorites = function (sid, contentType, ids, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	var items = ids.map(function (id) {
		return {
			id: id,
			contentType: contentType
		};
	});
	var data = {
		sessionID: sid,
		items: items
	};
	JsonServices.invoke(data, this, "DeleteFavorites", callback, errCallback);
};

/**
 * @function
 * @description Retrieves channel information matching client's request.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} type - Requests type of items. i.e. VOD, ArcPlus, etc. ({@link JsonServices.ContentType})
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetClientChannels = function (sid, type, onPage, pageNumber, callback, errCallback) {
	if (typeof(type) === "string") {
		type = JsonServices.ContentType[type];
	}
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var data = {
		sessionID: sid,
		request: new JsonServices.requests.ContentRequest(type, null, paging)
	};
	JsonServices.invoke(data, this, "GetClientChannels", callback, errCallback);
};

/**
 * @function
 * @description Retrieves a list of genres of a requested type for a specified client session.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} type - Definition of types of genres depending on the type of client services (such as Live TV, VOD etc.)
 * ({@link JsonServices.ContentGenreType})
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetClientContentGenres = function (sid, type, callback, errCallback) {
	if (typeof(type) === "string") {
		type = JsonServices.ContentGenreType[type];
	}
	var data = {
		sessionID: sid,
		type: type
	};
	JsonServices.invoke(data, this, "GetClientContentGenres", callback, errCallback);
};

/**
 * @function
 * @description Retrieves client accessible on-demand content. Valid for VOD and Arcplus content types.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} requestType - Defines search range for Video On Demand content (ArcPlus or VOD content type).
 * ({@link JsonServices.OnDemandRequestType})
 * @param {number} genreId - Genre id.
 * @param {string} keyWord - A word to look for in item names (i.e. names of films, TV programs, series etc.)
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetClientOnDemandContent = function (sid, requestType, genreId, keyWord, orderBy, onPage, pageNumber, callback, errCallback) {
	requestType = requestType || "AllItems";
	if (typeof(requestType) === "string") {
		requestType = JsonServices.OnDemandRequestType[requestType];
	}
	var contentType = JsonServices.ContentType["VOD"];
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var filter = JsonServices.requests.ContentFilter(contentType, true, genreId, null, null, null, orderBy, keyWord);
	var data = {
		sessionID: sid,
		request: JsonServices.requests.OnDemandRequest(contentType, requestType, filter, paging)
	};
	JsonServices.invoke(data, this, "GetClientOnDemandContent", callback, errCallback);
};

/**
 * @function
 * @description Retrieves a list of TV program items accessible to the client. Valid for DVR, LiveTV and ArcPlus content types.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} requestType - Defines search range for Video On Demand content (ArcPlus or VOD content type). ({@link JsonServices.OnDemandRequestType}).
 * @param {number} channelId - Requested items channel ID
 * @param {number} genreId - Genre id.
 * @param {string} dateFrom - Date of an item. Date when an item was added to the video collection.
 * @param {string} dateTill - Upper limit of item date for diapason search
 * @param {string} keyWord - A word to look for in item names (i.e. names of films, TV programs, series etc.)
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @param {boolean} isContainerSearch - Limit the search to "container" items only
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {string} streamZone - Stream time zone id to recalculate Program Guide for.
 * @param {string} watchingZone - Time zone for which TV Guide shows programmes timetable. Normally it is the time zone where client watches TV.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetClientProgramGuide = function (sid, contentType, requestType, channelId, genreId, dateFrom, dateTill, keyWord, orderBy, 
		isContainerSearch, onPage, pageNumber, streamZone, watchingZone, callback, errCallback) {
	requestType = requestType || "AllItems";
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	if (typeof(requestType) === "string") {
		requestType = JsonServices.OnDemandRequestType[requestType];
	}
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var filter = JsonServices.requests.ContentFilter(contentType, true, genreId, channelId, dateFrom, dateTill, orderBy, keyWord, isContainerSearch);
	var data = {
		sessionID: sid,
		type: contentType,
		request: JsonServices.requests.ProgramGuideRequest(contentType, requestType, dateFrom, dateTill, channelId, filter, paging, streamZone, watchingZone)
	};
	JsonServices.invoke(data, this, "GetClientProgramGuide", callback, errCallback);
};

/**
 * @function
 * @description Retrieves subsets of TV channels and their TV programs for forming the TV Guide Grid
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {string} dateFrom - Date of an item. Date when an item was added to the video collection.
 * @param {string} dateTill - Upper limit of item date for range search
 * @param {string} keyWord - A word to look for in item names (i.e. names of films, TV programs, series etc.)
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @param {boolean} isContainerSearch - Limit the search to "container" items only
 * @param {string} streamZone - Stream time zone id to recalculate Program Guide for.
 * @param {string} watchingZone - Time zone for which TV Guide shows programmes timetable. Normally it is the time zone where client watches TV.
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 * @param {boolean} showDetails - Flag indicating whether it is necessary to retrieve detailed TV program descriptions or not.
 */
JsonServices.content.GetTVGrid = function (sid, dateFrom, dateTill, streamZone, watchingZone, onPage, pageNumber, callback, errCallback, showDetails) {
	showDetails = typeof(showDetails) === "boolean" ? showDetails : true;
	
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var data = {
		sessionID: sid,
		request: {
			paging: paging,
			fromTime: dateFrom,
			tillTime: dateTill,
			streamZone: streamZone,
			watchingZone: watchingZone,
			showDetails: showDetails
		}
	};
	
	JsonServices.invoke(data, this, "GetTVGrid", callback, errCallback);
};

/**
 * @function
 * @description Retrieves list of TV program items related to the specified item accessible to the client. Valid for DVR, LiveTV and ArcPlus content types.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} relatedType - Type of related items to look for ({@link JsonServices.RelatedType})
 * @param {number} id - Id of the current item, for wich related items are being searched
 * @param {string} sortBy - Defines desired sort order of the search result set
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetClientRelatedProgramGuide = function (sid, contentType, relatedType, id, sortBy, onPage, pageNumber, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	if (typeof(relatedType) === "string") {
		relatedType = JsonServices.RelatedType[relatedType];
	}
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var filter = JsonServices.requests.ContentFilter(contentType, true, null, null, null, null, sortBy, null, null);
	var data = {
		sessionID: sid,
		request: JsonServices.requests.RelatedRequest(contentType, relatedType, id, filter, paging)
	};
	JsonServices.invoke(data, this, "GetClientRelatedProgramGuide", callback, errCallback);
};

/**
 * @function
 * @description Retrieves a list of related items of on-demand content that is accesible to the current client.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} relatedType - Type of related items to look for ({@link JsonServices.RelatedType})
 * @param {number} id - Id of the current item, for wich related items are being searched
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetClientRelatedOnDemandContent = function (sid, contentType, relatedType, id, orderBy, onPage, pageNumber, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	if (typeof(relatedType) === "string") {
		relatedType = JsonServices.RelatedType[relatedType];
	}
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var filter = JsonServices.requests.ContentFilter(contentType, true, null, null, null, null, orderBy);
	var data = {
		sessionID: sid,
		request: JsonServices.requests.RelatedRequest(contentType, relatedType, id, filter, paging)
	};
	JsonServices.invoke(data, this, "GetClientRelatedOnDemandContent", callback, errCallback);
};

/**
 * @function
 * @description Retrieves a list of related items of on-demand content that is accesible to the current client.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetFavPrograms = function (sid, contentType, onPage, pageNumber, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	var data = {
		sessionID: sid,
		type: contentType,
		paging: JsonServices.requests.ItemPaging(onPage, pageNumber)
	};
	JsonServices.invoke(data, this, "GetFavPrograms", callback, errCallback);
};

/**
 * @function
 * @description Retrieves a list of client's favorite on-demand items of VOD content type.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetFavOnDemandContent = function (sid, contentType, onPage, pageNumber, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	var data = {
		sessionID: sid,
		type: contentType,
		paging: JsonServices.requests.ItemPaging(onPage, pageNumber)
	};
	JsonServices.invoke(data, this, "GetFavOnDemandContent", callback, errCallback);
};

/**
 * @function
 * @description Performes search in client's favorite items of VOD content type
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} onPage - Desired number of items per page.
 * @param {number} pageNumber - Requested page number.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.content.GetFavOnDemandContentFilter = function (sid, contentType, onPage, pageNumber, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	var requestType = JsonServices.OnDemandRequestType["AllItems"];
	var paging = JsonServices.requests.ItemPaging(onPage, pageNumber);
	var data = {
		sessionID: sid,
		vodRequest: JsonServices.requests.OnDemandRequest(contentType, requestType, null, paging)
	};
	JsonServices.invoke(data, this, "GetFavOnDemandContentFilter", callback, errCallback);
};

/**
 * @function
 * @description Retrieves any stream URL for a logged in client. User specific.
 * @param {string} sid - Client's session ID received from {@link Login} method
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} id - Item unique id.
 * @param {number} baId - Unique broadcasting area id.
 * @param {number} cdnId - Unique delivery network id.
 * @param {number} qualityPreset - Media stream quality setting ({@link JsonServices.StreamQualityPreset}).
 * @param {string} shiftTimeZoneName - Broadcast shift time zone.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.media.GetClientStreamUri = function (sid, contentType, id, baId, cdnId, qualityPreset, shiftTimeZoneName, callback, errCallback) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	if (typeof(qualityPreset) === "string") {
		qualityPreset = JsonServices.StreamQualityPreset[qualityPreset];
	}
	var data = {
		sessionID: sid,
		mediaRequest: JsonServices.requests.MediaRequest(contentType, id, baId, cdnId, qualityPreset, shiftTimeZoneName)
	};
	JsonServices.invoke(data, this, "GetClientStreamUri", callback, errCallback);
};

/**
 * @function
 * @description Retrieves current utc time
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.media.GetCurrentUtcTime = function (callback, errCallback) {
	JsonServices.invoke({}, this, "GetCurrentUtcTime", callback, errCallback);
};

/**
 * @function
 * @description Retrieves all IVS Media time zones with info
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.media.GetMediaZoneInfo = function (callback, errCallback) {
	JsonServices.invoke({}, this, "GetMediaZoneInfo", callback, errCallback);
};

/**
 * @function
 * @description Retrieves url template for media.
 * @param {number} siteId - Id of a website via which client's request was processed.
 * @param {function} callback - event that occurs when all are done.
 * @param {function} errCallback - event that occurs when error is handle.
 */
JsonServices.media.MediaImageUrlTemplate = function (siteId, callback, errCallback) {
	siteId = siteId || 1;
	var data = { siteID: siteId };
	JsonServices.invoke(data, this, "MediaImageUrlTemplate", callback, errCallback);
};


/*================================================================*/

/** Requests */

/**
 * @function
 * @description Retrieves data structure containing data used for identification of the application that requests 
 * access to IVSMedia Service, and for transfering application settings between the service and the application.
 * @param {string} appName - The name of the application which is provided for application developer 
 * by IPTV-DISTRIBUTION This name is used as a part of login process for application identification.
 * @param {number} siteID - Unique ID provided for application developer by IPTV-DISTRIBUTION. 
 * It is used as a part of login process for application identification. 
 * This ID is necessary if you are developing a server application wich serves multiple users.
 * @returns {object} appSettings.
 */

JsonServices.requests.AppSettings = function (appName, siteID) {
	var appSettings = {
		appName: appName,
		siteID: siteID || 0,
		settings: []
	};
	
	return appSettings;
};

/**
 * @function
 * @description Retrieves data structure containing data used for authentification of the end user who requests 
 * IVSMedia services, and for transfering user settings between the service and the application.
 * @param {string} login - End client name for authentication.
 * @param {string} password - End client password for authentication.
 * @param {string} stbMacAddress - Hardware identifier of an appliance such as STB, smart TV etc.
 * @param {string} sessionId - Session identifier.
 * @param {string} clientIp - Client's IP address.
 * @returns {object} accessCredentials.
 */
JsonServices.requests.AccessCredentials = function (login, password, stbMacAddress, sessionId, clientIp) {
	var accessCredentials = {};
	if (login) {
		accessCredentials.UserLogin = login;
	}
	if (password) {
		accessCredentials.UserPassword = password;
	}
	if (stbMacAddress) {
		accessCredentials.StbMacAddress = stbMacAddress;
	}
	if (sessionId) {
		accessCredentials.sessionID = sessionId;
	}
	if (clientIp) {
		accessCredentials.clientIP = clientIp;
	}
	
	return accessCredentials;
};

/**
 * @function
 * @description Retrieves filters to be used in content item requests
 * @param {number} contentType - Id of the requested content type (i.e. LiveTV, VOD etc.) ({@link JsonServices.ContentType})
 * @param {boolean} availableOnly - Do not include items unavailable for the client to the search results.
 * @param {number} genreId - Genre id
 * @param {number} channelId - TV Channel id (applies to ArcPlus content type only)
 * @param {string} date - Date of an item.
 * @param {string} dateTill - Upper limit of item date for diapason search
 * @param {string} orderBy - Defines desired sort order of the search result set
 * @param {string} keyWord - A word to look for in item names (i.e. names of films, TV programs, series etc.)
 * @param {boolean} isContainerSearch - Limit the search to "container" items only
 * @returns {object} contentFilter.
 */
JsonServices.requests.ContentFilter = function (contentType, availableOnly, genreId, channelId, date, dateTill, orderBy, keyWord, isContainerSearch) {
	var contentFilter = {
		DontFetchDetails: contentType === JsonServices.ContentType.LiveTV,
		availableOnly: availableOnly,
		contentType: contentType,
		visibleOnly: true
	};
	if (isContainerSearch) {
		contentFilter.IsContainerSearch = isContainerSearch;
	}
	if (genreId) {
		genreId = parseInt(genreId);
		if (genreId) {
			contentFilter.contentGenre = {
				genreType: JsonServices.contentTypeGenreTypeMap[contentType],
				id: genreId
			};
		}
	}
	
	if (date) {
		contentFilter.date = date;
	}
	if (dateTill) {
		contentFilter.dateTill = dateTill;
	}
	if (keyWord) {
		contentFilter.keyWord = keyWord;
	}
	if (orderBy) {
		contentFilter.orderBy = orderBy;
	}
	if (channelId) {
		contentFilter.studioID = channelId;
	}
	
	return contentFilter;
};

/**
 * @function
 * @description Retrieves data structure for specifying the set of items sent during information retreival method calls.
 * @param {number} itemsOnPage - Desired number of items per page
 * @param {number} pageNumber - Requested page number.
 * @returns {object} paging.
 */
JsonServices.requests.ItemPaging = function (itemsOnPage, pageNumber) {
	if (!itemsOnPage && !pageNumber) {
		return null;
	}
	var paging = {
		itemsOnPage: itemsOnPage,
		pageNumber: pageNumber
	};
	
	return paging;
};

/**
 * @function
 * @description Retrieves data stracture with configuration settings that determine content delivery parameters.
 * @param {number} baId - Unique broadcasting area id
 * @param {number} cdnId - Unique delivery network id
 * @param {number} qualityPreset - Media stream quality setting ({@link JsonServices.StreamQualityPreset})
 * @param {string} shiftTimeZoneName - Broadcast shift time zone.
 * @returns {object} streamSettings.
 */
JsonServices.requests.StreamSettings = function (baId, cdnId, qualityPreset, shiftTimeZoneName) {
	var streamSettings = {};
	if (baId) {
		streamSettings.balancingArea = { id: baId };
	}
	if (cdnId) {
		streamSettings.cdn = { id: cdnId };
	}
	if (qualityPreset) {
		streamSettings.qualityPreset = qualityPreset;
	}
	if (shiftTimeZoneName) {
		streamSettings.shiftTimeZoneName = shiftTimeZoneName
	}
	
	return streamSettings;
};

/**
 * @function
 * @description Retrieves data structure containing client setting relating to media stream delivery and service availability.
 * @param {string} language - Client's user interface language preference.
 * @param {string} timeZoneName - Time zone for which TV Guide shows programmes timetable. Normally it is the time zone where client watches TV.
 * @param {string} shiftTimeZoneName - Broadcast shift time zone.
 * @param {number} qualityPreset - Media stream quality setting ({@link JsonServices.StreamQualityPreset})
 * @returns {object} clientSettings.
 */
JsonServices.requests.ClientSettings = function (language, timeZoneName, shiftTimeZoneName, qualityPreset) {
	var clientSettings = {};
	if (language) {
		clientSettings.language = language;
	}
	if (timeZoneName) {
		clientSettings.timeZoneName = timeZoneName;
	}
	if (shiftTimeZoneName || qualityPreset) {
		clientSettings.streamPreference = JsonServices.requests.StreamSettings(null, null, qualityPreset, shiftTimeZoneName);
	}
	
	return clientSettings;
};

/**
 * @function
 * @description Retrieves data structure representing a register information.
 * @param {number} type - registration type (Activate, Confirm for ex.) ({@link JsonServices.RegisterInfoType})
 * @param {string} email - A ContentFilter object defining search criteria for content request.
 * @param {string} pin - pin code.
 * @param {string} phoneNumber - phone number.
 * @returns {object} registerInfo.
 */
JsonServices.requests.RegisterInfo = function (type, email, pin, phoneNumber) {
	if (typeof(type) == "string") {
		type = JsonServices.RegisterInfoType[type];
	}
	var registerInfo = {};
	if (email) {
		registerInfo.EMail = email;
	}
	if (pin) {
		registerInfo.PIN = pin;
	}
	if (phoneNumber) {
		registerInfo.PhoneNumber = phoneNumber;
	}
	registerInfo.RegisterInfoType = type;
	
	return registerInfo;
};

/**
 * @function
 * @description Retrieves data structure representing a request for content items.
 * @param {object} type - Requeste type of items. i.e. VOD, ArcPlus, etc. 
 * @param {object} filter - A ContentFilter object defining search criteria for content request.
 * @param {object} paging - Paging specification of the search result set.
 * @returns {object} contentRequest.
 */
JsonServices.requests.ContentRequest = function (type, filter, paging) {
	if (typeof(type) === "string") {
		type = JsonServices.ContentType[type];
	}
	var contentRequest = {
		type: type
	};
	if (filter) {
		contentRequest.filter = filter;
	}
	if (paging) {
		contentRequest.paging = paging;
	}
	
	return contentRequest;
};

/**
 * @function
 * @description Retrieves application/website accessible on-demand content. Valid for VOD and Arcplus content types.
 * @param {object} type - Requeste type of items. i.e. VOD, ArcPlus, etc. 
 * @param {number} requestType - Defines search range for Video On Demand content (ArcPlus or VOD content type). ({@link JsonServices.OnDemandRequestType})
 * @param {object} filter - A ContentFilter object defining search criteria for content request.
 * @param {object} paging - Paging specification of the search result set.
 * @returns {object} onDemandRequest.
 */
JsonServices.requests.OnDemandRequest = function (type, requestType, filter, paging) {
	var onDemandRequest = JsonServices.requests.ContentRequest(type, filter, paging);
	if (requestType) {
		onDemandRequest.requestType = requestType;
	}
	return onDemandRequest;
};

/**
 * @function
 * @description Retrieves object for Program Guide items. Valid with DVR, LiveTV and ArcPlus content types.
 * @param {object} type - Requeste type of items. i.e. VOD, ArcPlus, etc. 
 * @param {number} requestType - Defines search range for Video On Demand content (ArcPlus or VOD content type). ({@link JsonServices.OnDemandRequestType})
 * @param {string} fromTime - Obsolete.
 * Request start time. This param is obsolete; use date field of content filter instead.
 * @param {string} tillTime - Obsolete.
 * Request end time. This param is obsolete; use dateTill field of content filter instead.
 * @param {number} channelId - Requested items channel ID
 * @param {object} filter - A ContentFilter object defining search criteria for content request.
 * @param {object} paging - Paging specification of the search result set.
 * @param {string} streamZone - Stream time zone id to recalculate Program Guide for
 * @param {string} watchingZone - Time zone for which TV Guide shows programmes timetable. Normally it is the time zone where client watches TV.
 * @returns {object} programGuideRequest.
 */
JsonServices.requests.ProgramGuideRequest = function (type, requestType, fromTime, tillTime, channelId, filter, paging, streamZone, watchingZone) {
	var programGuideRequest = JsonServices.requests.OnDemandRequest(type, requestType, filter, paging);
	if (channelId || channelId === 0) {
		programGuideRequest.channelID = channelId;
	}
	if (fromTime && tillTime) {
		programGuideRequest.fromTime = fromTime;
		programGuideRequest.tillTime = tillTime;
	}
	if (streamZone) {
		programGuideRequest.streamZone = streamZone;
	}
	if (watchingZone) {
		programGuideRequest.watchingZone = watchingZone;
	}
	
	return programGuideRequest;
};

/**
 * @function
 * @description Retrieves all information in order to request content stream URL
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} id - Item unique id.
 * @param {number} baId - Unique broadcasting area id.
 * @param {number} cdnId - Unique delivery network id.
 * @param {number} qualityPreset - Media stream quality setting ({@link JsonServices.StreamQualityPreset}).
 * @param {string} shiftTimeZoneName - Broadcast shift time zone.
 * @returns {object} mediaRequest.
 */

JsonServices.requests.MediaRequest = function (contentType, id, baId, cdnId, qualityPreset, shiftTimeZoneName) {
	if (!contentType || !id) {
		return null;
	}
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	if (typeof(qualityPreset) === "string") {
		qualityPreset = JsonServices.StreamQualityPreset[qualityPreset];
	}
	if (qualityPreset)
	var mediaRequest = {
		item: {
			id: id,
			contentType: contentType
		},
		streamSettings: JsonServices.requests.StreamSettings(baId, cdnId, qualityPreset, shiftTimeZoneName)
	};
	
	return mediaRequest;
};

/**
 * @function
 * @description Retrieves data structure representing request for items related to a specific item.
 * @param {number} contentType - Predefined list of possible subscription services ({@link JsonServices.ContentType}).
 * @param {number} relatedType - Type of related items to look for ({@link JsonServices.RelatedType})
 * @param {number} id - Item unique id.
 * @param {object} filter - A ContentFilter object defining search criteria for content request.
 * @param {object} paging - Paging specification of the search result set.
 * @returns {object} relatedRequest.
 */
JsonServices.requests.RelatedRequest = function (contentType, relatedType, id, filter, paging) {
	if (typeof(contentType) === "string") {
		contentType = JsonServices.ContentType[contentType];
	}
	if (typeof(relatedType) === "string") {
		relatedType = JsonServices.RelatedType[relatedType];
	}
	var relatedRequest = JsonServices.requests.ContentRequest(contentType, filter, paging);
	relatedRequest.id = id;
	relatedRequest.relatedType = relatedType;
	
	return relatedRequest;
};

/** /Requests */