/**
 * @author EvgeniyV (Profit-Labs Ltd.)
 */
 
/**
 * @class Class that describes login entity and provides work with it.
 */
Login = {};

/**
 * @function 
 * @describes Provides users login.
 * @param {string} DUID - Hardware identifier of an appliance such as STB, smart TV etc.
 * @param {string} login - End client name for authentication.
 * @param {string} passwd - End client password for authentication.
 * @param {function} successCb - event that occurs when all are done.
 * @param {function} errCb - event that occurs when error is handle.
 */
Login.completeLogin = function (DUID, login, passwd, successCb, errCb) {
    var successLogin = function () {
		Logger.write("Login successful, DUID: " + DUID, "LoginModule", true);
        Login.getClientSettings(successCb, errCb);
    };
    
    try {
        Login.login(DUID, login, passwd, successLogin, errCb);
    } catch (error) {
        errCb();
    }

};

/**
 * @function 
 * @describes Try to login with selected login and password
 * @param {string} DUID - Hardware identifier of an appliance such as STB, smart TV etc.
 * @param {string} login - End client name for authentication.
 * @param {string} passwd - End client password for authentication.
 * @param {function} successCb - event that occurs when all are done.
 * @param {function} errCb - event that occurs when error is handle.
 */
Login.login = function (DUID, login, passwd, successCb, errCb) {
    Logger.write("=========1===========Login: " + login + "  Password: " + passwd, "Loginjs");
	Logger.write("Login.login()", "LoginModule", true);
    var success = successCb || function(){};
    var err = errCb || function(){};
    var tmpLoginCallback = function (result) {
        try {
			if (result) {
				var credentials = result.clientCredentials;
				var appSettings = result.appSettings;
				AppContext.user.login = credentials.UserLogin;
				AppContext.user.password = credentials.UserPassword;
				AppContext.user.sid = credentials.sessionID;
				AppContext.user.userSystemPassword = credentials.UserSystemPassword;
				AppContext.user.siteId = appSettings.siteID;
				Logger.write("siteId: " + AppContext.user.siteId, "Login", true);
				if (AppContext.user.sid.length > 0) {
					success();
				} else {
					err(Resources.getResources().SceneLoginWrongLoginError);
				}
			} else {
				  err(Resources.getResources().SceneLoginLoginError);
			}
        } catch (error) {
			err(error);
        }
    };
    var errorCallback = function (jqXHR, textStatus, error) {
		//err(ErrorHandler.HandleError(error, "Login", true, textStatus));  
		err(error, textStatus);
    };
	
	JsonServices.client.Login(AppContext.appName, DUID, login, passwd, tmpLoginCallback, errorCallback);
};

/**
 * @function 
 * @describes Gets clients settings from web service
 * @param {function} successCb - event that occurs when all are done.
 * @param {function} errCb - event that occurs when error is handle.
 */
Login.getClientSettings = function (successCb, errCb) {
    var success = successCb || function(){};
    var err = errCb || function(){};
    
    var callbackIvs = function (result) {
		AppContext.ClientSettings.services = result.clientServices || [];
		AppContext.ClientSettings.shiftTimeZoneName = result.streamPreference.shiftTimeZoneName || "";
		AppContext.ClientSettings.bAId = result.streamPreference.balancingArea.id || "";
		AppContext.ClientSettings.cdnId = result.streamPreference.cdn.id || "";
		var qp = result.streamPreference.qualityPreset;
		for (var key in JsonServices.StreamQualityPreset) {
			if (qp == JsonServices.StreamQualityPreset[key]) {
				AppContext.ClientSettings.qualityPreset = key;
				break;
			}
		}
		AppContext.ClientSettings.timeZoneName = result.timeZoneName || ""; 
		
		var lang = false;
		Logger.write("Language: " + result.language, "Loginjs", true);
		for (var i = 0; i < AppContext.Languages.length; i++) {
			if(result.language == AppContext.Languages[i].id) {
				AppContext.ClientSettings.Language = AppContext.Languages[i];
				lang = true;
			}
		}
		if (!lang) {
			AppContext.ClientSettings.Language = AppContext.Languages[0];
		}
		Resources.getResources(AppContext.ClientSettings.Language.cultureInfo);
		
		JsonServices.media.MediaImageUrlTemplate(
			AppContext.user.siteId,
			function (result) {
				AppContext.ImageUrlTemplate = result;
				JsonServices.media.GetMediaZoneInfo(
					function (result) {
						var timeZones = Models.TimeZonesFromCollection(result);
						AppContext.loadTimeZones(timeZones);
						success();
						Updater.checkUpdate();
				}, errCallback);
			}, errCallback);
    };
    
    var errCallback = function (jqXHR, textStatus, error)
    {
		err(ErrorHandler.HandleError(error, "Login", true, textStatus, jqXHR));  
    };
    
	JsonServices.client.GetSettings(AppContext.user.sid, callbackIvs, errCallback);
};
